@@ -22,7 +22,7 @@ if ($config{'passwd_new'}) {
 	}
 if ($config{'passwd_dict'} && $_[0] =~ /^[A-Za-z\'\-]+$/ &&
     (&has_command("ispell") || &has_command("spell"))) {
-	local $temp = &tempname();
+	local $temp = &transname();
 	open(TEMP, ">$temp");
 	print TEMP $_[0],"\n";
 	close(TEMP);
@@ -44,6 +44,15 @@ if ($config{'passwd_dict'} && $_[0] =~ /^[A-Za-z\'\-]+$/ &&
 	unlink($temp);
 	return &text('change_epasswd_dict') if (!$unknown);
 	}
+if ($config{'passwd_prog'}) {
+        # Run external program with username and password as args
+        local $qu = quotemeta($_[1]);
+        local $qp = quotemeta($_[0]);
+        local $out = `$config{'passwd_prog'} $qu $qp 2>&1 </dev/null`;
+        if ($?) {
+                return $out;
+                }
+        }
 return undef;
 }
 
