@@ -10,19 +10,28 @@ $env_support = $config{'vixie_cron'};
 if ($module_info{'usermin'}) {
 	$single_user = $remote_user;
 	&switch_to_remote_user();
+	&create_user_config_dirs();
+	$range_cmd = "$user_module_config_directory/range.pl";
 	}
-$cron_temp_file = &tempname();
+else {
+	$range_cmd = "$module_config_directory/range.pl";
+	}
+$cron_temp_file = &transname();
+use Time::Local;
 
 # list_cron_jobs()
 # Returns a lists of structures of all cron jobs
 sub list_cron_jobs
 {
 local (@rv, $lnum, $f);
+if (defined(@cron_jobs_cache)) {
+	return @cron_jobs_cache;
+	}
 
 # read the master crontab file
 if ($config{'system_crontab'}) {
 	$lnum = 0;
-	open(TAB, $config{'system_crontab'});
+	&open_readfile(TAB, $config{'system_crontab'});
 	while(<TAB>) {
 		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(\S+)\s+(.*)/i) {
 			# A normal h m s d w time
@@ -61,11 +70,11 @@ if ($config{'system_crontab'}) {
 	}
 
 # read package-specific cron files
-opendir(DIR, $config{'cronfiles_dir'});
+opendir(DIR, &translate_filename($config{'cronfiles_dir'}));
 while($f = readdir(DIR)) {
 	next if ($f =~ /^\./);
 	$lnum = 0;
-	open(TAB, "$config{'cronfiles_dir'}/$f");
+	&open_readfile(TAB, "$config{'cronfiles_dir'}/$f");
 	while(<TAB>) {
 		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(\S+)\s+(.*)/i) {
 			push(@rv, { 'file' => "$config{'cronfiles_dir'}/$f",
@@ -96,6 +105,42 @@ while($f = readdir(DIR)) {
 	}
 closedir(DIR);
 
+# Read a single user's crontab file
+if ($config{'single_file'}) {
+	&open_readfile(TAB, $config{'single_file'});
+	$lnum = 0;
+	while(<TAB>) {
+		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(.*)/i) {
+			# A normal m h d m wd time
+			push(@rv, { 'file' => $config{'single_file'},
+				    'line' => $lnum,
+				    'type' => 3,
+				    'active' => !$1, 'nolog' => $2,
+				    'mins' => $3, 'hours' => $4,
+				    'days' => $5, 'months' => $6,
+				    'weekdays' => $8, 'user' => "NONE",
+				    'command' => $10,
+				    'index' => scalar(@rv) });
+			&fix_names($rv[$#rv]);
+			}
+		elsif (/^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*'([^']*)'/ ||
+		       /^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*"([^']*)"/ ||
+		       /^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*(\S+)/) {
+			# An environment variable
+			push(@rv, { 'file' => $config{'single_file'},
+				    'line' => $lnum,
+				    'active' => !$1,
+				    'name' => $2,
+				    'value' => $3,
+				    'user' => "NONE",
+				    'index' => scalar(@rv) });
+			}
+		$lnum++;
+		}
+	close(TAB);
+	}
+
+
 # read per-user cron files
 local $fcron = ($config{'cron_dir'} =~ /\/fcron$/);
 local @users;
@@ -103,17 +148,23 @@ if ($single_user) {
 	@users = ( $single_user );
 	}
 else {
-	opendir(DIR, $config{'cron_dir'});
+	opendir(DIR, &translate_filename($config{'cron_dir'}));
 	@users = grep { !/^\./ } readdir(DIR);
 	closedir(DIR);
 	}
 foreach $f (@users) {
 	next if (!(@uinfo = getpwnam($f)));
 	$lnum = 0;
-	open(TAB, $single_user ? "$config{'cron_user_get_command'} |" :
-		  $fcron ?
-		  &user_sub($config{'cron_get_command'}, $f)." |" :
-		  "$config{'cron_dir'}/$f");
+	if ($single_user) {
+		&open_execute_command(TAB, $config{'cron_user_get_command'}, 1);
+		}
+	elsif ($fcron) {
+		&open_execute_command(TAB,
+			&user_sub($config{'cron_get_command'}, $f), 1);
+		}
+	else {
+		&open_readfile(TAB, "$config{'cron_dir'}/$f");
+		}
 	while(<TAB>) {
 		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(.*)/i) {
 			# A normal m h d m wd time
@@ -126,6 +177,7 @@ foreach $f (@users) {
 				    'weekdays' => $8, 'user' => $f,
 				    'command' => $10,
 				    'index' => scalar(@rv) });
+			$rv[$#rv]->{'file'} =~ s/\s+\|$//;
 			&fix_names($rv[$#rv]);
 			}
 		elsif (/^(#+)?\s*@([a-z]+)\s+(.*)/i) {
@@ -156,7 +208,8 @@ foreach $f (@users) {
 	close(TAB);
 	}
 closedir(DIR);
-return @rv;
+@cron_jobs_cache = @rv;
+return @cron_jobs_cache;
 }
 
 sub cron_job_line
@@ -180,21 +233,30 @@ else {
 			 $_[0]->{'hours'}, $_[0]->{'days'},
 			 $_[0]->{'months'}, $_[0]->{'weekdays'});
 		}
-	push(@c, $_[0]->{'user'}) if ($_[0]->{'type'});
+	push(@c, $_[0]->{'user'}) if ($_[0]->{'type'} != 0 &&
+				      $_[0]->{'type'} != 3);
 	push(@c, $_[0]->{'command'});
 	}
 return join(" ", @c);
 }
 
 # copy_cron_temp(&job)
+# Copies a job's user's current cron configuration to the temp file
 sub copy_cron_temp
 {
+local $fcron = ($config{'cron_dir'} =~ /\/fcron$/);
 unlink($cron_temp_file);
 if ($single_user) {
-	system("$config{'cron_user_get_command'} >$cron_temp_file 2>/dev/null");
+	&execute_command($config{'cron_user_get_command'},
+			 undef, $cron_temp_file, undef);
+	}
+elsif ($fcron) {
+	&execute_command(&user_sub($config{'cron_get_command'},$_[0]->{'user'}),
+			 undef, $cron_temp_file, undef);
 	}
 else {
-	system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file 2>/dev/null");
+	system("cp ".&translate_filename("$config{'cron_dir'}/$_[0]->{'user'}").
+	       " $cron_temp_file 2>/dev/null");
 	}
 }
 
@@ -202,24 +264,80 @@ else {
 # Add a Cron job to a user's file
 sub create_cron_job
 {
-&copy_cron_temp($_[0]);
-local $lref = &read_file_lines($cron_temp_file);
-push(@$lref, &cron_job_line($_[0]));
-&flush_file_lines();
-system("chown $_[0]->{'user'} $cron_temp_file");
-&copy_crontab($_[0]->{'user'});
+&list_cron_jobs();	# init cache
+if ($config{'single_file'} && !$config{'cron_dir'}) {
+	# Add to the single file
+	$_[0]->{'type'} = 3;
+	local $lref = &read_file_lines($config{'single_file'});
+	push(@$lref, &cron_job_line($_[0]));
+	&flush_file_lines($config{'single_file'});
+	}
+else {
+	# Add to the specified user's crontab
+	&copy_cron_temp($_[0]);
+	local $lref = &read_file_lines($cron_temp_file);
+	$_[0]->{'line'} = scalar(@$lref);
+	push(@$lref, &cron_job_line($_[0]));
+	&flush_file_lines();
+	system("chown $_[0]->{'user'} $cron_temp_file");
+	&copy_crontab($_[0]->{'user'});
+	$_[0]->{'file'} = "$config{'cron_dir'}/$_[0]->{'user'}";
+	$_[0]->{'index'} = scalar(@cron_jobs_cache);
+	push(@cron_jobs_cache, $_[0]);
+	}
 }
 
 # insert_cron_job(&job)
 # Add a Cron job at the top of the user's file
 sub insert_cron_job
 {
-&copy_cron_temp($_[0]);
-local $lref = &read_file_lines($cron_temp_file);
-splice(@$lref, 0, 0, &cron_job_line($_[0]));
-&flush_file_lines();
-system("chown $_[0]->{'user'} $cron_temp_file");
-&copy_crontab($_[0]->{'user'});
+&list_cron_jobs();	# init cache
+if ($config{'single_file'} && !$config{'cron_dir'}) {
+	# Insert into single file
+	$_[0]->{'type'} = 3;
+	local $lref = &read_file_lines($config{'single_file'});
+	splice(@$lref, 0, 0, &cron_job_line($_[0]));
+	&flush_file_lines($config{'single_file'});
+	}
+else {
+	# Insert into the user's crontab
+	&copy_cron_temp($_[0]);
+	local $lref = &read_file_lines($cron_temp_file);
+	$_[0]->{'line'} = 0;
+	splice(@$lref, 0, 0, &cron_job_line($_[0]));
+	&flush_file_lines();
+	system("chown $_[0]->{'user'} $cron_temp_file");
+	&copy_crontab($_[0]->{'user'});
+	$_[0]->{'file'} = "$config{'cron_dir'}/$_[0]->{'user'}";
+	$_[0]->{'index'} = scalar(@cron_jobs_cache);
+	&renumber($_[0]->{'file'}, $_[0]->{'line'}, 1);
+	push(@cron_jobs_cache, $_[0]);
+	}
+}
+
+# renumber(file, line, offset)
+# All jobs in this file whose line is at or after the given one will be
+# incremented by the offset
+sub renumber
+{
+local $j;
+foreach $j (@cron_jobs_cache) {
+	if ($j->{'line'} >= $_[1] &&
+	    $j->{'file'} eq $_[0]) {
+		$j->{'line'} += $_[2];
+		}
+	}
+}
+
+# renumber_index(index, offset)
+sub renumber_index
+{
+local $j;
+foreach $j (@cron_jobs_cache) {
+	if ($j->{'index'} >= $_[0]) {
+		$j->{'index'} += $_[1];
+		}
+	}
 }
 
 # change_cron_job(&job)
@@ -248,6 +366,9 @@ if ($_[0]->{'type'} == 0) {
 else {
 	&replace_file_line($_[0]->{'file'}, $_[0]->{'line'});
 	}
+@cron_jobs_cache = grep { $_ ne $_[0] } @cron_jobs_cache;
+&renumber($_[0]->{'file'}, $_[0]->{'line'}, -1);
+&renumber_index($_[0]->{'index'}, -1);
 }
 
 # read_crontab(user)
@@ -255,7 +376,7 @@ else {
 sub read_crontab
 {
 local(@tab);
-open(TAB, "$config{cron_dir}/$_[0]");
+&open_readfile(TAB, "$config{cron_dir}/$_[0]");
 @tab = <TAB>;
 close(TAB);
 if (@tab >= 3 && $tab[0] =~ /DO NOT EDIT/ &&
@@ -267,20 +388,26 @@ return @tab;
 
 
 # copy_crontab(user)
+# Copy the cron temp file to that for this user
 sub copy_crontab
 {
+if (&is_readonly_mode()) {
+	# Do nothing
+	return undef;
+	}
 local($pwd);
 if (`cat $cron_temp_file` =~ /\S/) {
-	local $temp = &tempname();
+	local $temp = &transname();
 	local $rv;
 	if ($config{'cron_edit_command'}) {
 		# fake being an editor
-		local $notemp = &tempname();
-		open(NO, ">$notemp");
-		print NO "No\n";
-		print NO "N\n";
-		print NO "no\n";
-		close(NO);
+		# XXX does not work in translated command mode!
+		local $notemp = &transname();
+		&open_tempfile(NO, ">$notemp");
+		&print_tempfile(NO, "No\n");
+		&print_tempfile(NO, "N\n");
+		&print_tempfile(NO, "no\n");
+		&close_tempfile(NO);
 		$ENV{"VISUAL"} = $ENV{"EDITOR"} =
 			"$module_root_directory/cron_editor.pl";
 		$ENV{"CRON_EDITOR_COPY"} = $cron_temp_file;
@@ -299,13 +426,14 @@ if (`cat $cron_temp_file` =~ /\S/) {
 	else {
 		# use the cron copy command
 		if ($single_user) {
-			$rv = system($config{'cron_user_copy_command'}.
-				     " <$cron_temp_file >$temp 2>&1");
+			$rv = &execute_command(
+				$config{'cron_user_copy_command'},
+				$cron_temp_file, $temp, $temp);
 			}
 		else {
-			$rv = system(
-				&user_sub($config{'cron_copy_command'},$_[0]).
-				" <$cron_temp_file >$temp 2>&1");
+			$rv = &execute_command(
+				&user_sub($config{'cron_copy_command'}, $_[0]),
+				$cron_temp_file, $temp, $temp);
 			}
 		}
 	local $out = `cat $temp`;
@@ -318,12 +446,11 @@ if (`cat $cron_temp_file` =~ /\S/) {
 else {
 	# No more cron jobs left, so just delete
 	if ($single_user) {
-		system($config{'cron_user_delete_command'}.
-		       " >/dev/null 2>/dev/null");
+		&execute_command($config{'cron_user_delete_command'});
 		}
 	else {
-		system(&user_sub($config{'cron_delete_command'},$_[0]).
-		       " >/dev/null 2>/dev/null");
+		&execute_command(&user_sub(
+			$config{'cron_delete_command'}, $_[0]));
 		}
 	}
 unlink($cron_temp_file);
@@ -360,7 +487,7 @@ return $tmp;
 sub list_allowed
 {
 local(@rv, $_);
-open(ALLOW, $config{cron_allow_file});
+&open_readfile(ALLOW, $config{cron_allow_file});
 while(<ALLOW>) {
 	next if (/^\s*#/);
 	chop; push(@rv, $_) if (/\S/);
@@ -375,7 +502,7 @@ return @rv;
 sub list_denied
 {
 local(@rv, $_);
-open(DENY, $config{cron_deny_file});
+&open_readfile(DENY, $config{cron_deny_file});
 while(<DENY>) {
 	next if (/^\s*#/);
 	chop; push(@rv, $_) if (/\S/);
@@ -390,9 +517,11 @@ return @rv;
 sub save_allowed
 {
 local($_);
-open(ALLOW, ">$config{cron_allow_file}");
-foreach (@_) { print ALLOW $_,"\n"; }
-close(ALLOW);
+&open_tempfile(ALLOW, ">$config{cron_allow_file}");
+foreach (@_) {
+	&print_tempfile(ALLOW, $_,"\n");
+	}
+&close_tempfile(ALLOW);
 chmod(0444, $config{cron_allow_file});
 }
 
@@ -402,9 +531,11 @@ chmod(0444, $config{cron_allow_file});
 sub save_denied
 {
 local($_);
-open(DENY, "> $config{cron_deny_file}");
-foreach (@_) { print DENY $_,"\n"; }
-close(DENY);
+&open_tempfile(DENY, "> $config{cron_deny_file}");
+foreach (@_) {
+	&print_tempfile(DENY, $_,"\n");
+	}
+&close_tempfile(DENY);
 chmod(0444, $config{cron_deny_file});
 }
 
@@ -429,7 +560,7 @@ sub save_envs
 {
 local($i, @tab, $line);
 @tab = &read_crontab($_[0]);
-open(TAB, "> $cron_temp_file");
+open(TAB, ">$cron_temp_file");
 for($i=1; $i<@_; $i+=2) {
 	print TAB "$_[$i]=$_[$i+1]\n";
 	}
@@ -447,7 +578,7 @@ sub expand_run_parts
 local $dir = $_[0];
 $dir = "$config{'run_parts_dir'}/$dir"
 	if ($config{'run_parts_dir'} && $dir !~ /^\//);
-opendir(DIR, $dir);
+opendir(DIR, &translate_filename($dir));
 local @rv = readdir(DIR);
 closedir(DIR);
 @rv = grep { !/^\./ } @rv;
@@ -635,6 +766,53 @@ else {
 	}
 }
 
+# show_range_input(&job)
+# Given a cron job, prints fields for selecting it's run date range
+sub show_range_input
+{
+local ($job) = @_;
+print &ui_oneradio("range_def", 1, $text{'range_all'}, !$job->{'start'}),
+      "<br>\n";
+print &ui_oneradio("range_def", 0, "", $job->{'start'}),"\n";
+print &text('range_start', &ui_date_input(
+	$job->{'start'}->[0], $job->{'start'}->[1], $job->{'start'}->[2],
+	"range_start_day", "range_start_month", "range_start_year")),"\n",
+      &date_chooser_button(
+	"range_start_day", "range_start_month", "range_start_year"),"\n";
+print &text('range_end', &ui_date_input(
+	$job->{'end'}->[0], $job->{'end'}->[1], $job->{'end'}->[2],
+	"range_end_day", "range_end_month", "range_end_year")),"\n",
+      &date_chooser_button(
+	"range_end_day", "range_end_month", "range_end_year"),"\n";
+}
+
+# parse_range_input(&job, &in)
+# Updates the job object with the specified date range. May call &error
+# for invalid inputs
+sub parse_range_input
+{
+local ($job, $in) = @_;
+if ($in->{'range_def'}) {
+	# No range used
+	delete($job->{'start'});
+	delete($job->{'end'});
+	}
+else {
+	# Validate and store range
+	foreach my $r ("start", "end") {
+		eval { timelocal(0, 0, 0, $in{'range_'.$r.'_day'},
+					  $in{'range_'.$r.'_month'}-1,
+					  $in{'range_'.$r.'_year'}-1900) };
+		if ($@) {
+			&error($text{'range_e'.$r}." ".$@);
+			}
+		$job->{$r} = [ $in{'range_'.$r.'_day'},
+			       $in{'range_'.$r.'_month'},
+			       $in{'range_'.$r.'_year'} ];
+		}
+	}
+}
+
 @cron_month = ( 'jan', 'feb', 'mar', 'apr', 'may', 'jun',
 		'jul', 'aug', 'sep', 'oct', 'nov', 'dec' );
 @cron_weekday = ( 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' );
@@ -669,20 +847,32 @@ $_[0]->{'weekdays'} = join(",", @wds);
 sub create_wrapper
 {
 local $perl_path = &get_perl_path();
-open(CMD, ">$_[0]");
-print CMD <<EOF;
+&open_tempfile(CMD, ">$_[0]");
+&print_tempfile(CMD, <<EOF
 #!$perl_path
 open(CONF, "$config_directory/miniserv.conf");
 while(<CONF>) {
-\$root = \$1 if (/^root=(.*)/);
-}
+        \$root = \$1 if (/^root=(.*)/);
+        }
 close(CONF);
 \$ENV{'WEBMIN_CONFIG'} = "$ENV{'WEBMIN_CONFIG'}";
 \$ENV{'WEBMIN_VAR'} = "$ENV{'WEBMIN_VAR'}";
-chdir("\$root/$_[1]");
-exec("\$root/$_[1]/$_[2]", \@ARGV);
 EOF
-close(CMD);
+	);
+if ($gconfig{'os_type'} eq 'windows') {
+	# On windows, we need to chdir to the drive first, and use system
+	&print_tempfile(CMD, "if (\$root =~ /^([a-z]:)/i) {\n");
+	&print_tempfile(CMD, "       chdir(\"\$1\");\n");
+	&print_tempfile(CMD, "       }\n");
+	&print_tempfile(CMD, "chdir(\"\$root/$_[1]\");\n");
+	&print_tempfile(CMD, "exit(system(\"\$root/$_[1]/$_[2]\", \@ARGV));\n");
+	}
+else {
+	# Can use exec on Unix systems
+	&print_tempfile(CMD, "chdir(\"\$root/$_[1]\");\n");
+	&print_tempfile(CMD, "exec(\"\$root/$_[1]/$_[2]\", \@ARGV) || die \"Failed to run \$root/$_[1]/$_[2] : \$!\";\n");
+	}
+&close_tempfile(CMD);
 chmod(0755, $_[0]);
 }
 
@@ -693,30 +883,32 @@ sub cron_file
 return $_[0]->{'file'} || "$config{'cron_dir'}/$_[0]->{'user'}";
 }
 
-# when_text(&job)
+# when_text(&job, [upper-case-first])
 # Returns a text string describing when a cron job is run
 sub when_text
 {
+local $pfx = $_[1] ? "uc" : "";
 if ($_[0]->{'special'}) {
-	return $text{'edit_special_'.$_[0]->{'special'}};
+	$pfx = $_[1] ? "" : "lc";
+	return $text{$pfx.'edit_special_'.$_[0]->{'special'}};
 	}
 elsif ($_[0]->{'mins'} eq '*' && $_[0]->{'hours'} eq '*' && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
-	return $text{'when_min'};
+	return $text{$pfx.'when_min'};
 	}
 elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} eq '*' && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
-	return &text('when_hour', $_[0]->{'mins'});
+	return &text($pfx.'when_hour', $_[0]->{'mins'});
 	}
 elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
-	return &text('when_day', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'});
+	return &text($pfx.'when_day', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'});
 	}
 elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} =~ /^\d+$/ && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
-	return &text('when_month', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $_[0]->{'days'});
+	return &text($pfx.'when_month', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $_[0]->{'days'});
 	}
 elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} =~ /^\d+$/) {
-	return &text('when_weekday', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $text{"day_".$_[0]->{'weekdays'}});
+	return &text($pfx.'when_weekday', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $text{"day_".$_[0]->{'weekdays'}});
 	}
 else {
-	return &text('when_cron', join(" ", $_[0]->{'mins'}, $_[0]->{'hours'}, $_[0]->{'days'}, $_[0]->{'months'}, $_[0]->{'weekdays'}));
+	return &text($pfx.'when_cron', join(" ", $_[0]->{'mins'}, $_[0]->{'hours'}, $_[0]->{'days'}, $_[0]->{'months'}, $_[0]->{'weekdays'}));
 	}
 }
 
@@ -763,5 +955,94 @@ else {
 	}
 }
 
+# find_cron_process(&job, [&procs])
+sub find_cron_process
+{
+local @procs;
+if ($_[1]) {
+	@procs = @{$_[1]};
+	}
+else {
+	&foreign_require("proc", "proc-lib.pl");
+	@procs = &proc::list_processes();
+	}
+local $rpd = &is_run_parts($_[0]->{'command'});
+local @exp = $rpd ? &expand_run_parts($rpd) : ();
+local $cmd = $exp[0] || $_[0]->{'command'};
+$cmd =~ s/^\s*\[.*\]\s+\&\&\s+//;
+$cmd =~ s/^\s*\[.*\]\s+\|\|\s+//;
+while($cmd =~ s/(\d*)(<|>)((\/\S+)|&\d+)\s*$//) { }
+$cmd =~ s/^\((.*)\)\s*$/$1/;
+$cmd =~ s/\s+$//;
+if ($config{'match_mode'} == 1) {
+	$cmd =~ s/\s.*$//;
+	}
+($proc) = grep { $_->{'args'} =~ /\Q$cmd\E/ &&
+		 (!$config{'match_user'} || $_->{'user'} eq $_[0]->{'user'}) }
+		@procs;
+if (!$proc && $cmd =~ /^$config_directory\/(.*\.pl)(.*)$/) {
+	# Must be a Webmin wrapper
+	$cmd = "$root_directory/$1$2";
+	($proc) = grep { $_->{'args'} =~ /\Q$cmd\E/ &&
+			 (!$config{'match_user'} ||
+			  $_->{'user'} eq $_[0]->{'user'}) }
+			@procs;
+	}
+return $proc;
+}
+
+# extract_input(command)
+# Given a command line cmd%input , returns the command and input parts
+sub extract_input
+{
+local ($cmd) = @_;
+$cmd =~ s/\\%/\0/g;
+local ($cmd, $input) = split(/\%/, $cmd, 2);
+$cmd =~ s/\0/%/g;
+$input =~ s/\0/%/g;
+return ($cmd, $input);
+}
+
+# convert_range(&job)
+# Given a cron job that uses range.pl , work out the date range and update
+# the job object command
+sub convert_range
+{
+local ($job) = @_;
+local ($cmd, $input) = &extract_input($job->{'command'});
+if ($cmd =~ /^\Q$range_cmd\E\s+(\d+)\-(\d+)\-(\d+)\s+(\d+)\-(\d+)\-(\d+)\s+(.*)$/) {
+	# Looks like a range command
+	$job->{'start'} = [ $1, $2, $3 ];
+	$job->{'end'} = [ $4, $5, $6 ];
+	$job->{'command'} = $7;
+	$job->{'command'} =~ s/\\(.)/$1/g;
+	if ($input) {
+		$job->{'command'} .= '%'.$input;
+		}
+	return 1;
+	}
+return 0;
+}
+
+# unconvert_range(&job)
+sub unconvert_range
+{
+if ($job->{'start'}) {
+	# Need to add range command
+	local ($cmd, $input) = &extract_input($job->{'command'});
+	$job->{'command'} = $range_cmd." ".join("-", @{$job->{'start'}})." ".
+					   join("-", @{$job->{'end'}})." ".
+					   quotemeta($cmd);
+	if ($input) {
+		$job->{'command'} .= '%'.$input;
+		}
+	delete($job->{'start'});
+	delete($job->{'end'});
+	&create_wrapper($range_cmd, $module_name, "range.pl");
+	return 1;
+	}
+return 0;
+}
+
 1;
 
