@@ -79,8 +79,9 @@ $rv = [	[ 'AccessFileName', 0, 5, 'virtual', undef, 5 ],
 	[ 'ScoreBoardFile', 0, 9, 'global', '1.2-2.0' ],
 	[ 'SendBufferSize', 0, 1, 'global', -2.0 ],
 	[ 'ServerAdmin', 0, 1, 'virtual' ],
-	[ 'ServerAlias', 1, 1, 'virtual virtualonly', 1.2 ],
-	[ 'ServerName', 0, 1, 'virtual' ],
+	$access{'names'} ? (
+		[ 'ServerAlias', 1, 1, 'virtual virtualonly', 1.2 ],
+		[ 'ServerName', 0, 1, 'virtual' ] ) : ( ),
 	[ 'ServerPath', 0, 5, 'virtual' ],
 	[ 'ServerType', 0, 9, 'global', -2.0 ],
 	[ 'ServerTokens', 0, 9, 'global', 1.3 ],
