@@ -25,11 +25,11 @@ return @rv;
 sub set_htaccess_files
 {
 local $f;
-open(HTACCESS, ">$user_module_config_directory/htlist");
+&open_tempfile(HTACCESS, ">$user_module_config_directory/htlist");
 foreach $f (@_) {
-	print HTACCESS $f,"\n";
+	&print_tempfile(HTACCESS, $f,"\n");
 	}
-close(HTACCESS);
+&close_tempfile(HTACCESS);
 }
 
 1;
