@@ -330,6 +330,7 @@ if (@_ > 1 && !$one && $remote_user && !$gconfig{'notabs'}) {
     <td width="6" nowrap><img src="$gconfig{'webprefix'}/images/nav/left.jpg" width="3" height="57"></td>~;
 
     foreach $c (@cats) {
+	local $catdesc = $text{'longcategory_'.$c};
         $t = $cats{$c};
            $inlist    = "false";
            foreach $testet (@available) {
@@ -341,24 +342,24 @@ if (@_ > 1 && !$one && $remote_user && !$gconfig{'notabs'}) {
            if ($inlist eq "true") {
 
               if ($c eq "") {
-                print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/others.jpg" width="43" height="44"><br>~;
+                print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/others.jpg" width="43" height="44" title="$catdesc"><br>~;
             &chop_font;
 
                           print qq~</center></td>
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~; 
               } elsif ($c eq "webmin") {
                if (@_ > 1) {
-               print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44" border=0><br>~;
+               print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44" border=0 title="$catdesc"><br>~;
             &chop_font;
                           print qq~</a></center></td>
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~;
                 } else {
-               print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44" border=0><br>~;            &chop_font;
+               print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44" border=0 title="$catdesc"><br>~;            &chop_font;
                           print qq~</center></td>
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~;
                 }
                } else {
-               print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44"><br>~;
+               print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/$c.jpg" width="43" height="44" title="$catdesc"><br>~;
 
             &chop_font;
 
@@ -367,7 +368,7 @@ if (@_ > 1 && !$one && $remote_user && !$gconfig{'notabs'}) {
               }
 
         } else {
-            print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/unknown.jpg" width="43" height="44"><br>~;
+            print qq~<td nowrap><center><img src="$gconfig{'webprefix'}/images/cats_over/unknown.jpg" width="43" height="44" title="$catdesc"><br>~;
 
             &chop_font;
 
@@ -378,14 +379,14 @@ if (@_ > 1 && !$one && $remote_user && !$gconfig{'notabs'}) {
         else {
             if ($inlist eq "true") {
               if ($c eq "") {
-                print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/others.jpg" width="43" height="44" border=0 alt="$t"><br>~;
+                print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/others.jpg" width="43" height="44" border=0 alt="$t" title="$catdesc"><br>~;
 
             &chop_font;
 
                 print qq~</a></center></td>
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~; 
               } else {
-               print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/$c.jpg" width="43" height="44" border=0 alt="$t"><br>~;
+               print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/$c.jpg" width="43" height="44" border=0 alt="$t" title="$catdesc"><br>~;
 
             &chop_font;
 
@@ -393,7 +394,7 @@ if (@_ > 1 && !$one && $remote_user && !$gconfig{'notabs'}) {
     <td width="17" nowrap><img src="$gconfig{'webprefix'}/images/nav/sep.jpg" width="17" height="57"></td>~;
               }
         } else {
-            print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/unknown.jpg" width="43" height="44" border=0 alt="$t"><br>~;
+            print qq~<td nowrap><center><a href=$gconfig{'webprefix'}/?cat=$c><img src="$gconfig{'webprefix'}/images/cats/unknown.jpg" width="43" height="44" border=0 alt="$t" title="$catdesc"><br>~;
 
             &chop_font;
 
@@ -474,11 +475,11 @@ print qq~
 	    $usercol = defined($gconfig{'cs_header'}) ||
 		   defined($gconfig{'cs_table'}) ||
 		   defined($gconfig{'cs_page'});
-	    print "<td bgcolor=#bae3ff>",
+	    print "<td bgcolor=#bae3ff valign=top>",
 	      "<img src=$gconfig{'webprefix'}/images/tabs/blue_left.jpg width=13 height=22 ",
 	      "alt=\"\">","</td>\n";
 	    print "<td bgcolor=#bae3ff>&nbsp;<b>$title</b>&nbsp;</td>\n";
-	    print "<td bgcolor=#bae3ff>",
+	    print "<td bgcolor=#bae3ff valign=top>",
 	      "<img src=$gconfig{'webprefix'}/images/tabs/blue_right.jpg width=19 height=22 ",
 	      " alt=\"\">","</td>\n";
 	    if ($_[9]) {
@@ -540,6 +541,7 @@ for($i=0; $i+1<@_; $i+=2) {
     elsif ($url =~ /^\?/ && $module_name) {
         $url = "/$module_name/$url";
         }
+    $url = "$gconfig{'webprefix'}$url" if ($url =~ /^\//);
     if ($i == 0) {
         print "&nbsp;<a href=\"$url\"><img alt=\"<-\" align=middle border=0 src=$gconfig{'webprefix'}/images/arrow.jpg></a>\n";
         }
@@ -583,13 +585,6 @@ if (!$_[$i]) {
 
 }
 
-#sub theme_error {
-
-#print "error";
-
-#}
-
-
 sub chop_font {
 
 if (!$current_lang_info->{'titles'} || $gconfig{'texttitles'}) {
@@ -664,7 +659,7 @@ sub theme_error
 &header($text{'error'}, "");
 print "<p>\n";
 print "<h3>",($main::whatfailed ? "$main::whatfailed : " : ""),@_,"</h3>\n";
-&footer();
+&footer($ENV{'HTTP_REFERER'}, $text{'error_previous'});
 }
 
 sub theme_ui_print_unbuffered_header
@@ -675,5 +670,20 @@ $main::printed_unbuffered_header = 1;
 &ui_print_header(@_);
 }
 
+sub theme_popup_header
+{
+print "<!doctype html public \"-//W3C//DTD HTML 3.2 Final//EN\">\n";
+print "<html>\n";
+print "<title>$_[0]</title>\n";
+print $_[1];
+print "</head>\n";
+print '<body bgcolor=#6696bc link=#000000 vlink=#000000 text=#000000 leftmargin="2" topmargin="2" marginwidth="2" marginheight="2" '.$_[2].'>';
+}
+
+sub theme_popup_footer
+{
+print "</body></html>\n";
+}
+
 1;
 
