@@ -28,6 +28,7 @@ if ($module_info{'usermin'}) {
 		chop($mysql_version = <VERSION>);
 		close(VERSION);
 		}
+	$max_dbs = $userconfig{'max_dbs'};
 	}
 else {
 	# Webmin connects to the database as the user specified in the per-user
@@ -46,6 +47,7 @@ else {
 		close(VERSION);
 		}
 	$cron_cmd = "$module_config_directory/backup.pl";
+	$max_dbs = $config{'max_dbs'};
 	}
 $authstr = &make_authstr();
 $master_db = 'mysql';
@@ -71,13 +73,19 @@ EOF
 $old_user_priv_cols = $mysql_version >= 4 ? 21 : 14;
 $old_db_priv_cols = $mysql_version >= 4 ? 12 : 10;
 
+# make_authstr([login], [pass], [host], [port], [sock])
 sub make_authstr
 {
-return ($config{'sock'} ? " -S $config{'sock'}" : "").
-       ($config{'host'} ? " -h $config{'host'}" : "").
-       ($config{'port'} ? " -P $config{'port'}" : "").
-       ($mysql_login ? " -u ".quotemeta($mysql_login) : "").
-       ($mysql_pass ? " -p".quotemeta($mysql_pass) : "");
+local $login = defined($_[0]) ? $_[0] : $mysql_login;
+local $pass = defined($_[1]) ? $_[1] : $mysql_pass;
+local $host = defined($_[2]) ? $_[2] : $config{'host'};
+local $port = defined($_[3]) ? $_[3] : $config{'port'};
+local $sock = defined($_[4]) ? $_[4] : $config{'sock'};
+return ($sock ? " -S $sock" : "").
+       ($host ? " -h $host" : "").
+       ($port ? " -P $port" : "").
+       ($login ? " -u ".quotemeta($login) : "").
+       ($pass ? " -p".quotemeta($pass) : "");
 }
 
 # is_mysql_running()
@@ -85,7 +93,7 @@ return ($config{'sock'} ? " -S $config{'sock'}" : "").
 # inaccessible without a password
 sub is_mysql_running
 {
-local $out = `$config{'mysqladmin'} $authstr status 2>&1`;
+local $out = `"$config{'mysqladmin'}" $authstr status 2>&1`;
 return $out =~ /uptime/i ? 1 :
        $out =~ /denied|password/i ? -1 : 0;
 }
@@ -94,7 +102,7 @@ return $out =~ /uptime/i ? 1 :
 # Returns a list of all databases
 sub list_databases
 {
-open(DBS, "$config{'mysqlshow'} $authstr |");
+open(DBS, "\"$config{'mysqlshow'}\" $authstr |");
 local $t = &parse_mysql_table(DBS);
 close(DBS);
 return map { $_->[0] } @{$t->{'data'}};
@@ -105,10 +113,10 @@ return map { $_->[0] } @{$t->{'data'}};
 sub list_tables
 {
 if ($_[0] =~ /_/) {
-	open(DBS, "$config{'mysqlshow'} $authstr ".quotemeta($_[0])." % 2>&1 |");
+	open(DBS, "\"$config{'mysqlshow'}\" $authstr ".quotemeta($_[0])." % 2>&1 |");
 	}
 else {
-	open(DBS, "$config{'mysqlshow'} $authstr ".quotemeta($_[0])." 2>&1 |");
+	open(DBS, "\"$config{'mysqlshow'}\" $authstr ".quotemeta($_[0])." 2>&1 |");
 	}
 local $t = &parse_mysql_table(DBS);
 close(DBS);
@@ -130,7 +138,7 @@ return map { $_->[0] } @{$t->{'data'}};
 # Returns a list of hashes detailing the structure of a table
 sub table_structure
 {
-local $s = &execute_sql($_[0], "desc ".&quotestr($_[1]));
+local $s = &execute_sql_safe($_[0], "desc ".&quotestr($_[1]));
 local (@rv, $r);
 local (%tp, $i);
 for($i=0; $i<@{$s->{'titles'}}; $i++) {
@@ -148,9 +156,18 @@ return @rv;
 }
 
 # execute_sql(database, command, [param, ...])
-# Executes some SQL and returns the results
+# Executes some SQL and returns the results, after checking for the user's
+# readonly status.
 sub execute_sql
 {
+return { } if (&is_readonly_mode());
+return &execute_sql_safe(@_);
+}
+
+# execute_sql_safe(database, command, [param, ...])
+# Executes some SQL and returns the results
+sub execute_sql_safe
+{
 local $sql = $_[1];
 $sql =~ s/\\/\\\\/g;
 @params = @_[2..$#_];
@@ -164,6 +181,10 @@ if ($driver_handle) {
 					     { });
 	$dbh || &error("DBI connect failed : ",$driver_handle->errstr);
 	local $cmd = $dbh->prepare($sql);
+	if (!$cmd) {
+		&error(&text('esql', "<tt>".&html_escape($_[1])."</tt>",
+			     "<tt>".&html_escape($dbh->errstr)."</tt>"));
+		}
 	if (!$cmd->execute(@params)) {
 		&error(&text('esql', "<tt>".&html_escape($_[1])."</tt>",
 			     "<tt>".&html_escape($dbh->errstr)."</tt>"));
@@ -180,7 +201,7 @@ if ($driver_handle) {
 	}
 else {
 	# Use the mysql command program
-	local $temp = &tempname();
+	local $temp = &transname();
 	if (@params) {
 		# Sub in ? parameters
 		local $p;
@@ -198,13 +219,12 @@ else {
 	open(TEMP, ">$temp");
 	print TEMP $sql,"\n";
 	close(TEMP);
-	open(DBS, "$config{'mysql'} $authstr -E -t ".quotemeta($_[0])." <$temp 2>&1 |");
+	open(DBS, "\"$config{'mysql'}\" $authstr -E -t ".quotemeta($_[0])." <$temp 2>&1 |");
 	local $t = &parse_mysql_vertical(DBS);
 	close(DBS);
 	unlink($temp);
 	if (!ref($t)) {
 		$t =~ s/^ERROR[^:]*://;
-		&error($sql);
 		&error(&text('esql', "<tt>".&html_escape($_[1])."</tt>",
 			    "<tt>".&html_escape($t)."</tt>"));
 		}
@@ -212,7 +232,7 @@ else {
 	}
 }
 
-# execute_sql_logged(database, command)
+# execute_sql_logged(database, command, param, ...)
 sub execute_sql_logged
 {
 &additional_log('sql', $_[0], $_[1]);
@@ -334,7 +354,8 @@ else {
 	}
 }
 
-# split_table(&titles, &links, &col1, &col2, ...)
+# split_table(&titles, &checkboxes, &links, &col1, &col2, ...)
+# Outputs a table that is split into two parts
 sub split_table
 {
 local $mid = int((@{$_[1]}+1) / 2);
@@ -351,8 +372,9 @@ foreach $s ([0, $mid-1], [$mid, @{$_[1]}-1]) {
 		}
 	for($i=$s->[0]; $i<=$s->[1]; $i++) {
 		print "<tr $cb>\n";
-		print "<td><a href='$_[1]->[$i]'>$_[2]->[$i]</a></td>\n";
-		for($j=3; $j<@_; $j++) {
+		print "<td width=5%>$_[1]->[$i]</td>\n";
+		print "<td><a href='$_[2]->[$i]'>$_[3]->[$i]</a></td>\n";
+		for($j=4; $j<@_; $j++) {
 			print "<td>$_[$j]->[$i]</td>\n";
 			}
 		print "</tr>\n";
@@ -404,14 +426,24 @@ return $rv;
 # quotestr(string)
 sub quotestr
 {
+local $rv = $_[0];
 if (&supports_quoting()) {
-	return "`$_[0]`";
+	return "`$rv`";
 	}
 else {
-	return $_[0];
+	return $rv;
 	}
 }
 
+# escapestr(string)
+# Returns a string with quotes escaped, for use in SQL
+sub escapestr
+{
+local $rv = $_[0];
+$rv =~ s/'/''/g;
+return $rv;
+}
+
 # supports_quoting()
 # Returns 1 if running mysql version 3.23.6 or later
 sub supports_quoting
@@ -466,8 +498,8 @@ return $_[0]->{'type'} =~ /(text|blob)$/i;
 # problem
 sub get_mysql_version
 {
-local $out = `$config{'mysql'} -V 2>&1`;
-${$_[0]} = $out;
+local $out = &backquote_command("\"$config{'mysql'}\" -V 2>&1");
+${$_[0]} = $out if ($_[0]);
 if ($out =~ /lib\S+\.so/) {
 	return -1;
 	}
@@ -501,7 +533,8 @@ local $cmd = $config{'backup_before_'.$_[4]};
 if ($cmd) {
 	$ENV{'BACKUP_FILE'} = $_[3];
 	local $h = $_[1];
-	local $out = `($cmd) 2>&1 </dev/null`;
+	local $out;
+	&execute_command($cmd, undef, \$out, \$out);
 	if ($h && $out) {
 		print $h $_[2] ? "<pre>".&html_escape($out)."</pre>" : $out;
 		}
@@ -515,7 +548,8 @@ local $cmd = $config{'backup_after_'.$_[4]};
 if ($cmd) {
 	$ENV{'BACKUP_FILE'} = $_[3];
 	local $h = $_[1];
-	local $out = `($cmd) 2>&1 </dev/null`;
+	local $out;
+	&execute_command($cmd, undef, \$out, \$out);
 	if ($h && $out) {
 		print $h $_[2] ? "<pre>".&html_escape($out)."</pre>" : $out;
 		}
@@ -623,5 +657,52 @@ if (@auto) {
 return @sql;
 }
 
+# execute_sql_file(database, file, [user, pass])
+# Executes some file of SQL commands, and returns the exit status and output
+sub execute_sql_file
+{
+if (&is_readonly_mode()) {
+	return (0, undef);
+	}
+local ($db, $file, $user, $pass) = @_;
+local $authstr = &make_authstr($user, $pass);
+local $cmd = "$config{'mysql'} $authstr -t ".quotemeta($db)." <".quotemeta($file);
+-r $file || return (1, "$file does not exist");
+local $out = &backquote_logged("$cmd 2>&1");
+return ($?, $? ? $out || "$cmd failed" : $out);
+}
+
+# start_mysql()
+# Starts the MySQL database server, and returns undef on success or an
+# error message on failure.
+sub start_mysql
+{
+local $temp = &transname();
+local $rv = &system_logged("($config{'start_cmd'}) >$temp 2>&1");
+local $out = `cat $temp`; unlink($temp);
+if ($rv || $out =~ /failed/i) {
+	return "<pre>$out</pre>";
+	}
+return undef;
+}
+
+# stop_mysql()
+# Halts the MySQL database server, and returns undef on success or an
+# error message on failure.
+sub stop_mysql
+{
+local $out;
+if ($config{'stop_cmd'}) {
+	$out = &backquote_logged("$config{'stop_cmd'} 2>&1");
+	}
+else {
+	$out = &backquote_logged("$config{'mysqladmin'} $authstr shutdown 2>&1");
+	}
+if ($? || $out =~ /failed/i) {
+	return "<pre>$out</pre>";
+	}
+return undef;
+}
+
 1;
 
