@@ -32,7 +32,7 @@ return ($3, $1, $4, $2);
 #  3 = User and group quotas
 sub quota_can
 {
-return $_[0]->[2] eq "ufs" ? 1 : 0;
+return $_[0]->[2] eq "ufs" && $_[0]->[1] ne $_[0]->[0] ? 1 : 0;
 }
 
 # quota_now(&mnttab, &fstab)
@@ -109,11 +109,13 @@ return $rv;
 # be 1 for user only, 2 for group only or 3 for user and group
 sub quotaon
 {
+return if (&is_readonly_mode());
 local($qf, $out);
 $qf = "$_[0]/quotas";
 if (!(-r $qf)) {
-	open(QUOTAFILE, "> $qf"); close(QUOTAFILE);
-	chmod(0600, $qf);
+	&open_tempfile(QUOTAFILE, ">$qf", 0, 1);
+	&close_tempfile(QUOTAFILE);
+	&set_ownership_permissions(undef, undef, 0600, $qf);
 	}
 $out = &backquote_logged("$config{'user_quotaon_command'} $_[0] 2>&1");
 if ($?) { return $out; }
@@ -124,6 +126,7 @@ return undef;
 # Turn off quotas for some filesystem
 sub quotaoff
 {
+return if (&is_readonly_mode());
 local($out);
 $out = &backquote_logged("$config{'user_quotaoff_command'} $_[0] 2>&1");
 if ($?) { return $out; }
