@@ -348,6 +348,8 @@ else
 		echo ""
 		exit 13
 	fi
+
+	# Ask the user if SSL should be used
 	if [ "$ssl" = "" ]; then
 		ssl=0
 		$perl -e 'use Net::SSLeay' >/dev/null 2>/dev/null
@@ -363,6 +365,14 @@ else
 		fi
 	fi
 
+	# Don't use SSL if missing Net::SSLeay
+	if [ "$ssl" = "1" ]; then
+		$perl -e 'use Net::SSLeay' >/dev/null 2>/dev/null
+		if [ $? != "0" ]; then
+			ssl=0
+		fi
+	fi
+
 	# Copy files to target directory
 	echo "***********************************************************************"
 	if [ "$wadir" != "$srcdir" ]; then
