@@ -13,8 +13,8 @@ local $rv;
 $rv .= "<table border $tabletags>\n";
 $rv .= "<tr $tb> <td><b>$heading</b></td> </tr>\n" if (defined($heading));
 $rv .= "<tr $cb> <td><table width=100%>\n";
-$ui_table_cols = $cols || 4;
-$ui_table_pos = 0;
+$main::ui_table_cols = $cols || 4;
+$main::ui_table_pos = 0;
 return $rv;
 }
 
@@ -26,15 +26,19 @@ return &theme_ui_table_end(@_) if (defined(&theme_ui_table_end));
 return "</table></td></tr></table>\n";
 }
 
-# ui_columns_start(&headings, [width-percent], [noborder], [&tdtags])
+# ui_columns_start(&headings, [width-percent], [noborder], [&tdtags], [heading])
 # Returns HTML for a multi-column table, with the given headings
 sub ui_columns_start
 {
 return &theme_ui_columns_start(@_) if (defined(&theme_ui_columns_start));
-local ($heads, $width, $noborder, $tdtags) = @_;
+local ($heads, $width, $noborder, $tdtags, $heading) = @_;
 local $rv;
 $rv .= "<table".($noborder ? "" : " border").
 		(defined($width) ? " width=$width%" : "").">\n";
+if ($heading) {
+	$rv .= "<tr $tb><td colspan=".scalar(@$heads).
+	       "><b>$heading</b></td></tr>\n";
+	}
 $rv .= "<tr $tb>\n";
 local $i;
 for($i=0; $i<@$heads; $i++) {
@@ -94,15 +98,24 @@ sub ui_form_end
 return &theme_ui_form_end(@_) if (defined(&theme_ui_form_end));
 local ($buttons, $width) = @_;
 local $rv;
-if ($buttons) {
+if ($buttons && @$buttons) {
 	$rv .= "<table".($width ? " width=$width" : "")."><tr>\n";
 	local $b;
 	foreach $b (@$buttons) {
-		$rv .= "<td".(!$width ? "" :
-			      $b eq $buttons->[0] ? " align=left" :
-			      $b eq $buttons->[@$buttons-1] ?
-				" align=right" : " align=center").">".
-		       &ui_submit($b->[1], $b->[0])."</td>\n";
+		if (ref($b)) {
+			$rv .= "<td".(!$width ? "" :
+				      $b eq $buttons->[0] ? " align=left" :
+				      $b eq $buttons->[@$buttons-1] ?
+					" align=right" : " align=center").">".
+			       &ui_submit($b->[1], $b->[0]).
+			       ($b->[2] ? " ".$b->[2] : "")."</td>\n";
+			}
+		elsif ($b) {
+			$rv .= "<td>$b</td>\n";
+			}
+		else {
+			$rv .= "<td>&nbsp;&nbsp;</td>\n";
+			}
 		}
 	$rv .= "</tr></table>\n";
 	}
@@ -110,36 +123,65 @@ $rv .= "</form>\n";
 return $rv;
 }
 
-# ui_textbox(name, value, size)
+# ui_textbox(name, value, size, [disabled?])
 # Returns HTML for a text input
 sub ui_textbox
 {
 return &theme_ui_textbox(@_) if (defined(&theme_ui_textbox));
-local ($name, $value, $size) = @_;
+local ($name, $value, $size, $dis) = @_;
 return "<input name=\"".&quote_escape($name)."\" ".
        "value=\"".&quote_escape($value)."\" ".
-       "size=$size>";
+       "size=$size ".($dis ? "disabled=true" : "").">";
 }
 
-# ui_upload(name, size)
+# ui_bytesbox(name, bytes, [size], [disabled?])
+# Returns HTML for entering a number of bytes, but with friendly kB/MB/GB
+# options. May truncate values to 2 decimal points!
+sub ui_bytesbox
+{
+local ($name, $bytes, $size, $dis) = @_;
+local $units = 1;
+if ($bytes >= 10*1024*1024*1024) {
+	$units = 1024*1024*1024;
+	}
+elsif ($bytes >= 10*1024*1024) {
+	$units = 1024*1024;
+	}
+elsif ($bytes >= 10*1024) {
+	$units = 1024;
+	}
+else {
+	$units = 1;
+	}
+if ($bytes ne "") {
+	$bytes = sprintf("%.2f", ($bytes*1.0)/$units);
+	$bytes =~ s/\.00$//;
+	}
+return &ui_textbox($name, $bytes, $size || 8, $dis)." ".
+       &ui_select($name."_units", $units,
+		 [ [ 1, "bytes" ], [ 1024, "kB" ], [ 1024*1024, "MB" ],
+		   [ 1024*1024*1024, "GB" ] ], undef, undef, undef, $dis);
+}
+
+# ui_upload(name, size, [disabled?])
 # Returns HTML for a file upload input
 sub ui_upload
 {
 return &theme_ui_upload(@_) if (defined(&theme_ui_upload));
-local ($name, $size) = @_;
+local ($name, $size, $dis) = @_;
 return "<input type=file name=\"".&quote_escape($name)."\" ".
-       "size=$size>";
+       "size=$size ".($dis ? "disabled=true" : "").">";
 }
 
-# ui_password(name, value, size)
+# ui_password(name, value, size, [disabled?])
 # Returns HTML for a password text input
 sub ui_password
 {
 return &theme_ui_password(@_) if (defined(&theme_ui_password));
-local ($name, $value, $size) = @_;
+local ($name, $value, $size, $dis) = @_;
 return "<input type=password name=\"".&quote_escape($name)."\" ".
        "value=\"".&quote_escape($value)."\" ".
-       "size=$size>";
+       "size=$size ".($dis ? "disabled=true" : "").">";
 }
 
 # ui_hidden(name, value)
@@ -152,144 +194,187 @@ return "<input type=hidden name=\"".&quote_escape($name)."\" ".
        "value=\"".&quote_escape($value)."\">";
 }
 
-# ui_select(name, value|&values, &options, [size], [multiple])
+# ui_select(name, value|&values, &options, [size], [multiple],
+#	    [add-if-missing], [disabled?], [javascript])
 # Returns HTML for a drop-down menu or multiple selection list
 sub ui_select
 {
 return &theme_ui_select(@_) if (defined(&theme_ui_select));
-local ($name, $value, $opts, $size, $multiple) = @_;
+local ($name, $value, $opts, $size, $multiple, $missing, $dis, $js) = @_;
 local $rv;
 $rv .= "<select name=\"".&quote_escape($name)."\"".
        ($size ? " size=$size" : "").
-       ($multiple ? " multiple" : "").">\n";
-local $o;
+       ($multiple ? " multiple" : "").
+       ($dis ? " disabled=true" : "")." ".$js.">\n";
+local ($o, %opt, $s);
 local %sel = ref($value) ? ( map { $_, 1 } @$value ) : ( $value, 1 );
 foreach $o (@$opts) {
 	$rv .= "<option value=\"".&quote_escape($o->[0])."\"".
 	       ($sel{$o->[0]} ? " selected" : "").">".
 	       ($o->[1] || $o->[0])."\n";
+	$opt{$o->[0]}++;
+	}
+foreach $s (keys %sel) {
+	if (!$opt{$s} && $missing) {
+		$rv .= "<option value=\"".&quote_escape($s)."\"".
+		       "selected>".($s eq "" ? "&nbsp;" : $s)."\n";
+		}
 	}
 $rv .= "</select>\n";
 return $rv;
 }
 
-# ui_radio(name, value, &options)
+# ui_radio(name, value, &options, [disabled?])
 # Returns HTML for a series of radio buttons
 sub ui_radio
 {
 return &theme_ui_radio(@_) if (defined(&theme_ui_radio));
-local ($name, $value, $opts) = @_;
+local ($name, $value, $opts, $dis) = @_;
 local $rv;
 local $o;
 foreach $o (@$opts) {
+	local $id = &quote_escape($name."_".$o->[0]);
 	$rv .= "<input type=radio name=\"".&quote_escape($name)."\" ".
                "value=\"".&quote_escape($o->[0])."\"".
-	       ($o->[0] eq $value ? " checked" : "")."> ".
-	       ($o->[1] || $o->[0])."\n";
+	       ($o->[0] eq $value ? " checked" : "").
+	       ($dis ? " disabled=true" : "").
+	       " id=\"$id\"".
+	       " $o->[2]> <label for=\"$id\">".
+	       ($o->[1] || $o->[0])."</label>\n";
 	}
 return $rv;
 }
 
-# ui_checkbox(name, value, label, selected?)
+# ui_yesno_radio(name, value, [yes], [no], [disabled?])
+# Like ui_yesno, but always displays just two inputs (yes and no)
+sub ui_yesno_radio
+{
+local ($name, $value, $yes, $no, $dis) = @_;
+$yes = 1 if (!defined($yes));
+$no = 0 if (!defined($no));
+return &theme_ui_yesno_radio(@_) if (defined(&theme_ui_yesno_radio));
+return &ui_radio($name, $value, [ [ $yes, $text{'yes'} ],
+				  [ $no, $text{'no'} ] ], $dis);
+}
+
+# ui_checkbox(name, value, label, selected?, [tags], [disabled?])
 # Returns HTML for a single checkbox
 sub ui_checkbox
 {
 return &theme_ui_checkbox(@_) if (defined(&theme_ui_checkbox));
-local ($name, $value, $label, $sel) = @_;
+local ($name, $value, $label, $sel, $tags, $dis) = @_;
+local $id = &quote_escape($name);
 return "<input type=checkbox name=\"".&quote_escape($name)."\" ".
        "value=\"".&quote_escape($value)."\" ".
-       ($sel ? " checked" : "")."> $label\n";
+       ($sel ? " checked" : "").($dis ? " disabled=true" : "").
+       " id=\"$id\"".
+       " $tags> <label for=\"$id\">$label</label>\n";
 }
 
-# ui_oneradio(name, value, label, selected?)
+# ui_oneradio(name, value, label, selected?, [tags], [disabled?])
 # Returns HTML for a single radio button
 sub ui_oneradio
 {
 return &theme_ui_oneradio(@_) if (defined(&theme_ui_oneradio));
-local ($name, $value, $label, $sel) = @_;
+local ($name, $value, $label, $sel, $tags, $dis) = @_;
+local $id = &quote_escape($name);
 return "<input type=radio name=\"".&quote_escape($name)."\" ".
        "value=\"".&quote_escape($value)."\" ".
-       ($sel ? " checked" : "")."> $label\n";
+       ($sel ? " checked" : "").($dis ? " disabled=true" : "").
+       " id=\"$id\"".
+       " $tags> <label for=\"$id\">$label</label>\n";
 }
 
-# ui_textarea(name, value, rows, cols, [wrap])
+# ui_textarea(name, value, rows, cols, [wrap], [disabled?])
 # Returns HTML for a multi-line text input
 sub ui_textarea
 {
 return &theme_ui_textarea(@_) if (defined(&theme_ui_textarea));
-local ($name, $value, $rows, $cols, $wrap) = @_;
+local ($name, $value, $rows, $cols, $wrap, $dis) = @_;
 return "<textarea name=\"".&quote_escape($name)."\" ".
-       "rows=$rows cols=$cols".($wrap ? " wrap=$wrap" : "").">".
+       "rows=$rows cols=$cols".($wrap ? " wrap=$wrap" : "").
+       ($dis ? " disabled=true" : "").">".
        &html_escape($value).
        "</textarea>";
 }
 
-# ui_user_textbox(name, value, [form])
+# ui_user_textbox(name, value, [form], [disabled?])
 # Returns HTML for a Unix user input
 sub ui_user_textbox
 {
 return &theme_ui_user_textbox(@_) if (defined(&theme_ui_user_textbox));
-return &unix_user_input($_[0], $_[1], $_[2]);
+return &ui_textbox($_[0], $_[1], 13, $_[3])." ".
+       &user_chooser_button($_[0], 1, $_[2]);
 }
 
-# ui_group_textbox(name, value, [form])
+# ui_group_textbox(name, value, [form], [disabled?])
 # Returns HTML for a Unix group input
 sub ui_group_textbox
 {
 return &theme_ui_group_textbox(@_) if (defined(&theme_ui_group_textbox));
-return &unix_group_input($_[0], $_[1], $_[2]);
+return &ui_textbox($_[0], $_[1], 13, $_[3])." ".
+       &group_chooser_button($_[0], 1, $_[2]);
 }
 
-# ui_opt_textbox(name, value, size, option1, [option2])
+# ui_opt_textbox(name, value, size, option1, [option2], [disabled?],
+#		 [&extra-fields])
 # Returns HTML for a text field that is optional
 sub ui_opt_textbox
 {
 return &theme_ui_opt_textbox(@_) if (defined(&theme_ui_opt_textbox));
-local ($name, $value, $size, $opt1, $opt2) = @_;
+local ($name, $value, $size, $opt1, $opt2, $dis, $extra) = @_;
+local $dis1 = &js_disable_inputs([ $name, @$extra ], [ ]);
+local $dis2 = &js_disable_inputs([ ], [ $name, @$extra ]);
 local $rv;
 $rv .= "<input type=radio name=\"".&quote_escape($name."_def")."\" ".
-       "value=1 ".($value ne '' ? "" : "checked")."> ".$opt1."\n";
+       "value=1 ".($value ne '' ? "" : "checked").
+       ($dis ? " disabled=true" : "")." onClick='$dis1'> ".$opt1."\n";
 $rv .= "<input type=radio name=\"".&quote_escape($name."_def")."\" ".
-       "value=0 ".($value ne '' ? "checked" : "")."> ".$opt2."\n";
+       "value=0 ".($value ne '' ? "checked" : "").
+       ($dis ? " disabled=true" : "")." onClick='$dis2'> ".$opt2."\n";
 $rv .= "<input name=\"".&quote_escape($name)."\" ".
-       "size=$size value=\"".&quote_escape($value)."\">\n";
+       "size=$size value=\"".&quote_escape($value)."\" ".
+       ($value eq "" || $dis ? "disabled=true" : "").">\n";
 return $rv;
 }
 
-# ui_submit(label, [name])
+# ui_submit(label, [name], [disabled?])
 # Returns HTML for a form submit button
 sub ui_submit
 {
 return &theme_ui_submit(@_) if (defined(&theme_ui_submit));
-local ($label, $name) = @_;
+local ($label, $name, $dis) = @_;
 return "<input type=submit".
        ($name ne '' ? " name=\"".&quote_escape($name)."\"" : "").
-       " value=\"".&quote_escape($label)."\">\n";
+       " value=\"".&quote_escape($label)."\"".
+       ($dis ? " disabled=true" : "").">\n";
 			
 }
 
-# ui_reset(label)
+# ui_reset(label, [disabled?])
 # Returns HTML for a form reset button
 sub ui_reset
 {
 return &theme_ui_reset(@_) if (defined(&theme_ui_reset));
-local ($label) = @_;
-return "<input type=submit value=\"".&quote_escape($label)."\">\n";
+local ($label, $dis) = @_;
+return "<input type=submit value=\"".&quote_escape($label).
+       ($dis ? " disabled=true" : "")."\">\n";
 			
 }
 
-# ui_date_input(day, month, year, day-name, month-name, year-name)
+# ui_date_input(day, month, year, day-name, month-name, year-name, [disabled?])
 # Returns HTML for a date-selection field
 sub ui_date_input
 {
-local ($day, $month, $year, $dayname, $monthname, $yearname) = @_;
+local ($day, $month, $year, $dayname, $monthname, $yearname, $dis) = @_;
 local $rv;
-$rv .= &ui_textbox($dayname, $day, 3);
+$rv .= &ui_textbox($dayname, $day, 3, $dis);
 $rv .= "/";
 $rv .= &ui_select($monthname, $month,
-		  [ map { [ $_, $text{"smonth_$_"} ] } (1 .. 12) ]);
+		  [ map { [ $_, $text{"smonth_$_"} ] } (1 .. 12) ],
+		  1, 0, 0, $dis);
 $rv .= "/";
-$rv .= &ui_textbox($yearname, $year, 5);
+$rv .= &ui_textbox($yearname, $year, 5, $dis);
 return $rv;
 }
 
@@ -302,11 +387,11 @@ return &theme_ui_table_row(@_) if (defined(&theme_ui_table_row));
 local ($label, $value, $cols) = @_;
 $cols ||= 1;
 local $rv;
-$rv .= "<tr>\n" if ($ui_table_pos%$ui_table_cols == 0);
+$rv .= "<tr>\n" if ($main::ui_table_pos%$main::ui_table_cols == 0);
 $rv .= "<td valign=top $_[3]->[0]><b>$label</b></td>\n" if (defined($label));
 $rv .= "<td valign=top colspan=$cols $_[3]->[1]>$value</td>\n";
-$ui_table_pos += $cols+(defined($label) ? 1 : 0);
-$rv .= "</tr>\n" if ($ui_table_pos%$ui_table_cols == 0);
+$main::ui_table_pos += $cols+(defined($label) ? 1 : 0);
+$rv .= "</tr>\n" if ($main::ui_table_pos%$main::ui_table_cols == 0);
 return $rv;
 }
 
@@ -315,7 +400,7 @@ sub ui_table_hr
 {
 return &theme_ui_table_hr(@_) if (defined(&theme_ui_table_hr));
 $ui_table_pos = 0;
-return "<tr> <td colspan=$ui_table_cols><hr></td> </tr>\n";
+return "<tr> <td colspan=$main::ui_table_cols><hr></td> </tr>\n";
 }
 
 # ui_buttons_start()
@@ -332,18 +417,26 @@ return &theme_(@_) if (defined(&theme_));
 return "</table>\n";
 }
 
-# ui_buttons_row(script, button-label, description, [hiddens], [after-submit]) 
+# ui_buttons_row(script, button-label, description, [hiddens], [after-submit],
+#		 [before-submit]) 
 sub ui_buttons_row
 {
 return &theme_ui_buttons_row(@_) if (defined(&theme_ui_buttons_row));
-local ($script, $label, $desc, $hiddens, $after) = @_;
+local ($script, $label, $desc, $hiddens, $after, $before) = @_;
 return "<form action=$script>\n".
        $hiddens.
-       "<tr> <td nowrap>".&ui_submit($label)." $after</td>\n".
-       "<td valign=top>$desc</td> </tr>\n".
+       "<tr> <td nowrap width=20%>".($before ? $before." " : "").
+       &ui_submit($label).($after ? " ".$after : "")."</td>\n".
+       "<td valign=top width=80%>$desc</td> </tr>\n".
        "</form>\n";
 }
 
+# ui_buttons_hr()
+sub ui_buttons_hr
+{
+return &theme_ui_buttons_hr(@_) if (defined(&theme_ui_buttons_hr));
+return "<tr> <td colspan=2><hr></td> </tr>\n";
+}
 
 ####################### header and footer functions
 
@@ -426,5 +519,44 @@ print $text,"<p>\n";
 exit;
 }
 
+# ui_subheading(text, ...)
+# Returns HTML for a section heading
+sub ui_subheading
+{
+return &theme_ui_subheading(@_) if (defined(&theme_ui_subheading));
+return "<h3>".join("", @_)."</h3>\n";
+}
+
+####################### javascript functions
+
+# js_disable_input(&disable-inputs, &enable-inputs, [tag])
+# Returns Javascript to disable some form elements and enable others
+sub js_disable_inputs
+{
+local $rv;
+local $f;
+foreach $f (@{$_[0]}) {
+	$rv .= "form.$f.disabled = true; ";
+	}
+foreach $f (@{$_[1]}) {
+	$rv .= "form.$f.disabled = false; ";
+	}
+return $_[2] ? "$_[2]='$rv'" : $rv;
+}
+
+# js_checkbox_disable(name, &checked-disable, &checked-enable, [tag])
+sub js_checkbox_disable
+{
+local $rv;
+local $f;
+foreach $f (@{$_[1]}) {
+	$rv .= "form.$f.disabled = $_[0].checked; ";
+	}
+foreach $f (@{$_[2]}) {
+	$rv .= "form.$f.disabled = !$_[0].checked; ";
+	}
+return $_[3] ? "$_[3]='$rv'" : $rv;
+}
+
 1;
 
