@@ -10,20 +10,20 @@ if (&has_command("mount.cifs")) {
 	$cifs_support = 4;
 	}
 if (&has_command("mount.smbfs")) {
-	$smbfs_support = `mount.smbfs -v` =~ /username=/i ? 4 : 3;
+	$smbfs_support = &backquote_command("mount.smbfs -v", 1) =~ /username=/i ? 4 : 3;
 	$smbfs_fs = "smbfs";
 	}
 elsif (&has_command("mount.smb")) {
-	$smbfs_support = `mount.smb -v` =~ /username=/i ? 4 : 3;
+	$smbfs_support = &backquote_command("mount.smb -v", 1) =~ /username=/i ? 4 : 3;
 	$smbfs_fs = "smb";
 	}
 elsif (&has_command("smbmount")) {
-	$smbfs_support = `smbmount -v` =~ /Version\s+2/i ? 2 : 1;
+	$smbfs_support = &backquote_command("smbmount -v", 1) =~ /Version\s+2/i ? 2 : 1;
 	$smbfs_fs = "smbfs";
 	}
 $swaps_support = -r "/proc/swaps";
 if (&has_command("e2label") &&
-    `mount --version 2>&1` =~ /(\d+)\.(\d+)/ && ($1 > 2 || $1==2 && $2 >= 10)) {
+    &backquote_command("mount --version 2>&1", 1) =~ /(\d+)\.(\d+)/ && ($1 > 2 || $1==2 && $2 >= 10)) {
 	$has_e2label = 1;
 	}
 if (`uname -r` =~ /^(\d+\.\d+)/ && $1 >= 2.4) {
@@ -62,7 +62,10 @@ while(<FSTAB>) {
 	if ($p[2] eq "proc") { $p[0] = "proc"; }
 	elsif ($p[2] eq "auto") { $p[2] = "*"; }
 	elsif ($p[2] eq "swap") { $p[1] = "swap"; }
-	elsif ($p[2] eq $smbfs_fs || $p[2] eq "cifs") { $p[0] =~ s/\//\\/g; }
+	elsif ($p[2] eq $smbfs_fs || $p[2] eq "cifs") {
+		$p[0] =~ s/\\040/ /g;
+		$p[0] =~ s/\//\\/g;
+		}
 	$rv[$i] = [ $p[1], $p[0], $p[2] ];
 	$rv[$i]->[5] = "yes";
 	@o = split(/,/ , $p[3] eq "defaults" ? "" : $p[3]);
@@ -157,27 +160,32 @@ if ($_[2] eq "auto") {
 	elsif ($amd_support == 2) {
 		# Adding a new automounter mount
 		local @amfs = split(/\s+/, $config{'auto_file'});
-		open(AMD, ">>$amfs[0]");
-		print AMD "\n";
-		print AMD "[ $_[0] ]\n";
-		print AMD "map_name = $_[1]\n";
-		close(AMD);
+		&open_tempfile(AMD, ">>$amfs[0]");
+		&print_tempfile(AMD, "\n");
+		&print_tempfile(AMD, "[ $_[0] ]\n");
+		&print_tempfile(AMD, "map_name = $_[1]\n");
+		&close_tempfile(AMD);
 		}
 	}
 elsif ($_[2] eq "autofs") {
 	# Adding a new automounter mount
-	open(AUTO, ">> $config{'autofs_file'}");
-	print AUTO "$_[0]  $_[1]";
-	if ($_[3]) { print AUTO "  ",&autofs_args($_[3]); }
-	print AUTO "\n";
-	close(AUTO);
+	&open_tempfile(AUTO, ">> $config{'autofs_file'}");
+	&print_tempfile(AUTO, "$_[0]  $_[1]");
+	if ($_[3]) {
+		&print_tempfile(AUTO, "  ",&autofs_args($_[3]));
+		}
+	&print_tempfile(AUTO, "\n");
+	&close_tempfile(AUTO);
 	}
 else {
 	# Adding a normal mount to the fstab file
 	local $dev = $_[1];
-	$dev =~ s/\\/\//g if ($_[2] eq $smbfs_fs || $_[2] eq "cifs");
-	open(FSTAB, ">> $config{fstab_file}");
-	print FSTAB "$dev  $_[0]  $_[2]";
+	if ($_[2] eq $smbfs_fs || $_[2] eq "cifs") {
+		$dev =~ s/\\/\//g;
+		$dev =~ s/ /\\040/g;
+		}
+	&open_tempfile(FSTAB, ">> $config{fstab_file}");
+	&print_tempfile(FSTAB, "$dev  $_[0]  $_[2]");
 	local @opts = $_[3] eq "-" ? ( ) : split(/,/, $_[3]);
 	if ($_[5] eq "no") {
 		push(@opts, "noauto");
@@ -188,11 +196,11 @@ else {
 	if ($_[2] eq "bind") {
 		push(@opts, "bind");
 		}
-	if (!@opts) { print FSTAB "  defaults"; }
-	else { print FSTAB "  ".join(",", @opts); }
-	print FSTAB "  0  ";
-	print FSTAB $_[4] eq "-" ? "0\n" : "$_[4]\n";
-	close(FSTAB);
+	if (!@opts) { &print_tempfile(FSTAB, "  defaults"); }
+	else { &print_tempfile(FSTAB, "  ".join(",", @opts)); }
+	&print_tempfile(FSTAB, "  0  ");
+	&print_tempfile(FSTAB, $_[4] eq "-" ? "0\n" : "$_[4]\n");
+	&close_tempfile(FSTAB);
 	}
 undef(@list_mounts_cache);
 }
@@ -209,12 +217,14 @@ $i = 0;
 open(FSTAB, $config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, "> $config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line =~ /\S/ && $line !~ /\signore\s/ && $i++ == $_[0]) {
 		# Found the line to replace
-		print FSTAB "$_[2]  $_[1]  $_[3]";
+		local $dev = $_[2];
+		$dev =~ s/ /\\040/g;
+		&print_tempfile(FSTAB, "$dev  $_[1]  $_[3]");
 		local @opts = $_[4] eq "-" ? ( ) : split(/,/, $_[4]);
 		if ($_[6] eq "no") {
 			push(@opts, "noauto");
@@ -225,14 +235,14 @@ foreach (@fstab) {
 		if ($_[3] eq "bind") {
 			push(@opts, "bind");
 			}
-		if (!@opts) { print FSTAB "  defaults"; }
-		else { print FSTAB "  ".join(",", @opts); }
-		print FSTAB "  0  ";
-		print FSTAB $_[5] eq "-" ? "0\n" : "$_[5]\n";
+		if (!@opts) { &print_tempfile(FSTAB, "  defaults"); }
+		else { &print_tempfile(FSTAB, "  ".join(",", @opts)); }
+		&print_tempfile(FSTAB, "  0  ");
+		&print_tempfile(FSTAB, $_[5] eq "-" ? "0\n" : "$_[5]\n");
 		}
-	else { print FSTAB $_,"\n"; }
+	else { &print_tempfile(FSTAB, $_,"\n"); }
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 
 if ($amd_support == 1) {
 	# Update older amd configuration
@@ -276,17 +286,21 @@ if ($autofs_support) {
 	open(AUTO, $config{'autofs_file'});
 	@auto = <AUTO>;
 	close(AUTO);
-	open(AUTO, "> $config{'autofs_file'}");
+	&open_tempfile(AUTO, "> $config{'autofs_file'}");
 	foreach (@auto) {
 		chop; ($line = $_) =~ s/#.*$//g;
 		if ($line =~ /\S/ && $i++ == $_[0]) {
-			print AUTO "$_[1]  $_[2]";
-			if ($_[4]) { print AUTO "  ",&autofs_args($_[4]); }
-			print AUTO "\n";
+			&print_tempfile(AUTO, "$_[1]  $_[2]");
+			if ($_[4]) {
+				&print_tempfile(AUTO, "  ",&autofs_args($_[4]));
+				}
+			&print_tempfile(AUTO, "\n");
+			}
+		else {
+			&print_tempfile(AUTO, $_,"\n");
 			}
-		else { print AUTO $_,"\n"; }
 		}
-	close(AUTO);
+	&close_tempfile(AUTO);
 	}
 undef(@list_mounts_cache);
 }
@@ -303,18 +317,19 @@ $i = 0;
 open(FSTAB, $config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
-open(FSTAB, "> $config{fstab_file}");
+&open_tempfile(FSTAB, ">$config{fstab_file}");
 foreach (@fstab) {
 	chop; ($line = $_) =~ s/#.*$//g;
 	if ($line !~ /\S/ || $line =~ /\signore\s/ || $i++ != $_[0]) {
 		# Don't delete this line
-		print FSTAB $_,"\n";
+		&print_tempfile(FSTAB, $_,"\n");
 		}
 	}
-close(FSTAB);
+&close_tempfile(FSTAB);
 
 if ($amd_support == 1) {
 	# Update older amd configuration
+	local $foundamd = 0;
 	local $amd = &read_amd_conf();
 	if ($amd =~ /MOUNTPTS='(.*)'/) {
 		# found mount points line..
@@ -322,12 +337,13 @@ if ($amd_support == 1) {
 		for($j=0; $j<@mpts; $j+=2) {
 			if ($i++ == $_[0]) {
 				splice(@mpts, $j, 2);
+				$foundamd = 1;
 				}
 			}
 		local $mpts = join(" ", @mpts);
 		$amd =~ s/MOUNTPTS='(.*)'/MOUNTPTS='$mpts'/;
 		}
-	&write_amd_conf($amd);
+	&write_amd_conf($amd) if ($foundamd);
 	}
 elsif ($amd_support == 2) {
 	# Update new amd configuration
@@ -348,7 +364,7 @@ if ($amd_support) {
 	open(AMD, $config{auto_file});
 	@amd = <AMD>;
 	close(AMD);
-	open(AMD, "> $config{auto_file}");
+	&open_tempfile(AMD, ">$config{auto_file}");
 	foreach (@amd) {
 		if (/MOUNTPTS='(.*)'/) {
 			# found mount points line..
@@ -359,11 +375,11 @@ if ($amd_support) {
 					push(@nmpts, $mpts[$j+1]);
 					}
 				}
-			print AMD "MOUNTPTS='".join(' ', @nmpts)."'\n";
+			&print_tempfile(AMD, "MOUNTPTS='".join(' ', @nmpts)."'\n");
 			}
-		else { print AMD $_; }
+		else { &print_tempfile(AMD, $_); }
 		}
-	close(AMD);
+	&close_tempfile(AMD);
 	}
 
 # Update autofs file
@@ -371,15 +387,15 @@ if ($autofs_support) {
 	open(AUTO, $config{'autofs_file'});
 	@auto = <AUTO>;
 	close(AUTO);
-	open(AUTO, "> $config{'autofs_file'}");
+	&open_tempfile(AUTO, ">$config{'autofs_file'}");
 	foreach (@auto) {
 		chop; ($line = $_) =~ s/#.*$//g;
 		if ($line !~ /\S/ || $i++ != $_[0]) {
 			# keep this line
-			print AUTO $_,"\n";
+			&print_tempfile(AUTO, $_,"\n");
 			}
 		}
-	close(AUTO);
+	&close_tempfile(AUTO);
 	}
 undef(@list_mounts_cache);
 }
@@ -413,7 +429,7 @@ while(<MTAB>) {
 	elsif ($p[2] eq "autofs") {
 		# Kernel automounter map.. use the pid to find the map
 		$p[0] =~ /automount\(pid(\d+)\)/ || next;
-		$out = `ps hwwww $1`;
+		$out = &backquote_command("ps hwwww $1", 1);
 		$out =~ /automount\s+(.*)\s*(\S+)\s+(file|program|yp)(,\S+)?\s+(\S+)/ || next;
 		$p[0] = $5;
 		$p[3] = $1 ? &autofs_options($1) : "-";
@@ -435,12 +451,12 @@ while(<MTAB>) {
 		# the device.
 		local $label;
 		if ($p[2] eq "xfs") {
-			local $out = `xfs_db -x -p xfs_admin -c label -r $p[0] 2>&1`;
+			local $out = &backquote_command("xfs_db -x -p xfs_admin -c label -r $p[0] 2>&1", 1);
 			$label = $1 if ($out =~ /label\s*=\s*"(.*)"/ &&
 					$1 ne '(null)');
 			}
 		else {
-			$label = `e2label $p[0] 2>&1`;
+			$label = &backquote_command("e2label $p[0] 2>&1", 1);
 			chop($label);
 			}
 		if (!$?) {
@@ -475,7 +491,7 @@ while(<SWAPS>) {
 	chop;
 	if (/^(\/\S+)\s+/) {
 		local $sf = $1;
-		if ($sf =~ /^\/dev\/ide\//) {
+		if ($sf =~ /^\/dev\/ide\// || $sf =~ /^\/dev\/mapper\//) {
 			# check fstab for a mount on a device which is a symlink
 			local @st = stat($sf);
 			foreach $m (@mounts) {
@@ -503,9 +519,17 @@ return @rv;
 sub mount_dir
 {
 local($out, $opts, $shar, %options, %smbopts);
+local @opts = $_[3] eq "-" || $_[3] eq "" ? ( ) :
+		grep { $_ ne "noauto" } split(/,/, $_[3]);
+if ($_[2] eq "bind") {
+	push(@opts, "bind");
+	}
+$opts = @opts ? "-o ".quotemeta(join(",", @opts)) : "";
+&parse_options($_[2], $_[3]);
 if ($_[2] eq "swap") {
 	# Use swapon to add the swap space..
-	$out = &backquote_logged("swapon $_[1] 2>&1");
+	local $priarg = $options{'pri'} ne "" ? "-p $options{'pri'}" : "";
+	$out = &backquote_logged("swapon $priarg $_[1] 2>&1");
 	if ($out =~ /Invalid argument/) {
 		# looks like this swap partition isn't ready yet.. set it up
 		$out = &backquote_logged("mkswap $_[1] 2>&1");
@@ -529,13 +553,14 @@ elsif ($_[2] eq "autofs") {
 	}
 elsif ($_[2] eq $smbfs_fs || $_[2] eq "cifs") {
 	local $shar = $_[1];
-	$shar =~ s/\\/\//g;
+	$shar =~ s/\\/\//g if ($shar =~ /^\\/);
 	local $support = $_[2] eq $smbfs_fs ? $smbfs_support : $cifs_support;
+	return uc($_[2])." not supported" if (!$support);
+	local $qshar = quotemeta($shar);
 	if ($support >= 3) {
 		# SMB filesystem mounted with mount command
-		$opts = $_[3] eq "-" ? "" : "-o '$_[3]'";
-		local $temp = &tempname();
-		local $ex = &system_logged("mount -t $_[2] $opts $shar $_[0] >$temp 2>&1 </dev/null");
+		local $temp = &transname();
+		local $ex = &system_logged("mount -t $_[2] $opts $qshar $_[0] >$temp 2>&1 </dev/null");
 		local $out = `cat $temp`;
 		unlink($temp);
 		if ($ex || $out =~ /failed|error/i) {
@@ -545,7 +570,6 @@ elsif ($_[2] eq $smbfs_fs || $_[2] eq "cifs") {
 		}
 	elsif ($support == 2) {
 		# SMB filesystem mounted with version 2.x smbmount
-		&parse_options($_[2], $_[3]);
 		$opts =
 		    ($options{'user'} ? "-U $options{'user'} " : "").
 		    ($options{'passwd'} ? "" : "-N ").
@@ -577,7 +601,6 @@ elsif ($_[2] eq $smbfs_fs || $_[2] eq "cifs") {
 		}
 	elsif ($support == 1) {
 		# SMB filesystem mounted with older smbmount
-		&parse_options($_[2], $_[3]);
 		$shortname = &get_system_hostname();
 		if ($shortname =~ /^([^\.]+)\.(.+)$/) { $shortname = $1; }
 		$opts =
@@ -603,9 +626,6 @@ elsif ($_[2] eq $smbfs_fs || $_[2] eq "cifs") {
 	}
 else {
 	# some filesystem supported by mount
-	local @o = $_[3] eq "-" ? ( ) : split(/,/, $_[3]);
-	push(@o, "bind") if ($_[2] eq "bind");
-	$opts = @o ? "-o \"".join(",", @o)."\"" : "";
 	local $fs = $_[2] eq "*" ? "auto" : $_[2];
 	if ($_[1] =~ /LABEL=(.*)/) {
 		$cmd = "mount -t $fs -L $1 $opts $_[0]";
@@ -620,6 +640,36 @@ undef(@list_mounted_cache);
 return 0;
 }
 
+# os_remount_dir(directory, device, type, options)
+# Adjusts the options for some mounted filesystem, by re-mounting
+sub os_remount_dir
+{
+if ($_[2] eq "swap" || $_[2] eq "auto" || $_[2] eq "autofs" ||
+    $_[2] eq $smbfs_fs || $_[2] eq "cifs") {
+	# Cannot use remount
+	local $err = &unmount_dir(@_);
+	return $err if ($err);
+	return &mount_dir(@_);
+	}
+else {
+	# Attempt to use remount
+	local @opts = $_[3] eq "-" || $_[3] eq "" ? ( ) :
+			grep { $_ ne "noauto" } split(/,/, $_[3]);
+	push(@opts, "remount");
+	local $opts = @opts ? "-o ".quotemeta(join(",", @opts)) : "";
+	local $fs = $_[2] eq "*" ? "auto" : $_[2];
+	if ($_[1] =~ /LABEL=(.*)/) {
+		$cmd = "mount -t $fs -L $1 $opts $_[0]";
+		}
+	else {
+		$cmd = "mount -t $fs $opts $_[1] $_[0]";
+		}
+	$out = &backquote_logged("$cmd 2>&1 </dev/null");
+	if ($?) { return "<pre>$out</pre>"; }
+	undef(@list_mounted_cache);
+	return undef;
+	}
+}
 
 # unmount_dir(directory, device, type)
 # Unmount a directory that is currently mounted. Returns 0 if ok,
@@ -635,7 +685,7 @@ if ($_[2] eq "swap") {
 elsif ($_[2] eq "auto") {
 	# Kill the amd process
 	$dir = $_[0];
-	if (`cat /etc/mtab` =~ /:\(pid([0-9]+)\)\s+$dir\s+(auto|nfs)\s+/) {
+	if (&backquote_command("cat /etc/mtab") =~ /:\(pid([0-9]+)\)\s+$dir\s+(auto|nfs)\s+/) {
 		&kill_logged('TERM', $1) || return $text{'linux_ekillamd'};
 		}
 	sleep(2);
@@ -643,7 +693,7 @@ elsif ($_[2] eq "auto") {
 elsif ($_[2] eq "autofs") {
 	# Kill the automount process
 	$dir = $_[0];
-	`cat /etc/mtab` =~ /automount\(pid([0-9]+)\)\s+$dir\s+autofs\s+/;
+	&backquote_command("cat /etc/mtab") =~ /automount\(pid([0-9]+)\)\s+$dir\s+autofs\s+/;
 	&kill_logged('TERM', $1) || return $text{'linux_ekillauto'};
 	sleep(2);
 	}
@@ -655,8 +705,8 @@ else {
 		&read_smbopts();
 		delete($smbopts{$_[0]});
 		&write_smbopts();
-		system("rmmod smbfs >/dev/null 2>&1 </dev/null");
-		system("rmmod cifs >/dev/null 2>&1 </dev/null");
+		&execute_command("rmmod smbfs");
+		&execute_command("rmmod cifs");
 		}
 	}
 undef(@list_mounted_cache);
@@ -713,10 +763,24 @@ sub disk_space
 if (&get_mounted($_[1], "*") < 0) { return (); }
 if ($_[0] eq "proc" || $_[0] eq "swap" ||
     $_[0] eq "auto" || $_[0] eq "autofs") { return (); }
-`df -k $_[1]` =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/;
-return ($1, $2);
+if (&backquote_command("LC_ALL='' LANG='' df -k $_[1]", 1) =~
+    /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
+	return ($1, $2);
+	}
+return ( );
 }
 
+# inode_space(type, directory)
+# Returns the total and free number of inodes for some filesystem.
+sub inode_space
+{
+if (&get_mounted($_[1], "*") < 0) { return (); }
+if (&backquote_command("LC_ALL='' LANG='' df -i $_[1]", 1) =~
+    /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/) {
+	return ($1, $2);
+	}
+return ( );
+}
 
 # list_fstypes()
 # Returns an array of all the supported filesystem types. If a filesystem is
@@ -724,8 +788,9 @@ return ($1, $2);
 # generate_options() will not be called for it.
 sub list_fstypes
 {
-local @sup = ("ext2", "minix", "msdos", "nfs", "nfs4", "iso9660", "ext", "xiafs",
-	      "hpfs", "fat", "vfat", "umsdos", "sysv", "reiserfs", "ntfs");
+local @sup = ("ext2", "minix", "msdos", "nfs", "nfs4", "iso9660", "ext",
+	      "xiafs", "hpfs", "fat", "vfat", "umsdos", "sysv", "reiserfs",
+	      "ntfs");
 push(@sup, $smbfs_fs) if ($smbfs_support);
 push(@sup, "cifs") if ($cifs_support);
 push(@sup, "auto") if ($amd_support);
@@ -837,12 +902,13 @@ elsif ($_[0] eq "swap") {
 elsif ($_[0] eq $smbfs_fs || $_[0] eq "cifs") {
 	# Windows filesystem
 	$_[1] =~ /^\\\\(.*)\\(.*)$/;
+	local ($server, $share) = ($1, $2);
 	print "<tr> <td><b>$text{'linux_smbserver'}</b></td>\n";
-	print "<td><input name=smbfs_server value=\"$1\" size=20>\n";
+	print "<td><input name=smbfs_server value=\"$server\" size=20>\n";
 	&smb_server_chooser_button("smbfs_server");
 	print "</td>\n";
 	print "<td><b>$text{'linux_smbshare'}</b></td>\n";
-	print "<td><input name=smbfs_share value=\"$2\" size=20>\n";
+	print "<td><input name=smbfs_share value=\"$share\" size=20>\n";
 	&smb_share_chooser_button("smbfs_server", "smbfs_share");
 	print "</td> </tr>\n";
 	}
@@ -1302,7 +1368,12 @@ elsif ($_[0] eq "autofs") {
 	}
 elsif ($_[0] eq "swap") {
 	# Swap has no options..
-	print "<tr> <td><i>$text{'linux_noopts'}</i></td> </tr>\n";
+	print "<tr> <td width=25%><b>$text{'linux_swappri'}</b></td>\n";
+	print "<td width=25%>",&ui_opt_textbox("swap_pri", $options{'pri'}, 6,
+				     $text{'default'}),"</td>\n";
+
+	print "<td width=50% colspan=2></td>\n";
+	print "</tr>\n";
 	}
 elsif ($_[0] eq $smbfs_fs || $_[0] eq "cifs") {
 	# SMB filesystems have a few options..
@@ -1320,108 +1391,119 @@ elsif ($_[0] eq $smbfs_fs || $_[0] eq "cifs") {
 	printf "<td><input type=password name=smbfs_passwd size=15 value=\"%s\"></td> </tr>\n",
 		$support == 4 ? $options{'password'} : $options{'passwd'};
 
-	if ($support != 2) {
-		print "<tr> <td><b>$text{'linux_uid'}</b></td>\n";
-		printf "<td><input name=smbfs_uid size=8 value=\"%s\">\n",
-			defined($options{"uid"}) ? getpwuid($options{"uid"}) : "";
-		print &user_chooser_button("smbfs_uid", 0),"</td>\n";
+	if (!$access{'simopts'}) {
+		if ($support != 2) {
+			print "<tr> <td><b>$text{'linux_uid'}</b></td>\n";
+			printf "<td><input name=smbfs_uid size=8 value=\"%s\">\n",
+				defined($options{"uid"}) ? getpwuid($options{"uid"}) : "";
+			print &user_chooser_button("smbfs_uid", 0),"</td>\n";
 
-		print "<td><b>$text{'linux_gid'}</b></td>\n";
-		printf "<td><input name=smbfs_gid size=8 value=\"%s\">\n",
-			defined($options{"gid"}) ? getgrgid($options{"gid"}) : "";
-		print &group_chooser_button("smbfs_gid", 0),"</td>\n";
-		}
+			print "<td><b>$text{'linux_gid'}</b></td>\n";
+			printf "<td><input name=smbfs_gid size=8 value=\"%s\">\n",
+				defined($options{"gid"}) ? getgrgid($options{"gid"}) : "";
+			print &group_chooser_button("smbfs_gid", 0),"</td>\n";
+			}
 
-	if ($support == 1) {
-		print "<tr> <td><b>$text{'linux_sname'}</b></td>\n";
-		printf "<td><input type=radio name=smbfs_sname_def value=1 %s> $text{'linux_auto'}\n",
-			defined($options{"servername"}) ? "" : "checked";
-		printf "<input type=radio name=smbfs_sname_def value=0 %s>\n",
-			defined($options{"servername"}) ? "checked" : "";
-		print "<input size=10 name=smbfs_sname value=\"$options{servername}\"></td>\n";
-		}
-	elsif ($support == 2) {
-		print "<tr> <td><b>$text{'linux_wg'}</b></td>\n";
-		printf "<td><input type=radio name=smbfs_wg_def value=1 %s> $text{'linux_auto'}\n",
-			defined($options{"workgroup"}) ? "" : "checked";
-		printf "<input type=radio name=smbfs_wg_def value=0 %s>\n",
-			defined($options{"workgroup"}) ? "checked" : "";
-		print "<input size=10 name=smbfs_wg value=\"$options{'workgroup'}\"></td>\n";
-		}
+		if ($support == 1) {
+			print "<tr> <td><b>$text{'linux_sname'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_sname_def value=1 %s> $text{'linux_auto'}\n",
+				defined($options{"servername"}) ? "" : "checked";
+			printf "<input type=radio name=smbfs_sname_def value=0 %s>\n",
+				defined($options{"servername"}) ? "checked" : "";
+			print "<input size=10 name=smbfs_sname value=\"$options{servername}\"></td>\n";
+			}
+		elsif ($support == 2) {
+			print "<tr> <td><b>$text{'linux_wg'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_wg_def value=1 %s> $text{'linux_auto'}\n",
+				defined($options{"workgroup"}) ? "" : "checked";
+			printf "<input type=radio name=smbfs_wg_def value=0 %s>\n",
+				defined($options{"workgroup"}) ? "checked" : "";
+			print "<input size=10 name=smbfs_wg value=\"$options{'workgroup'}\"></td>\n";
+			}
 
-	if ($support < 3) {
-		print "<td><b>$text{'linux_cname'}</b></td>\n";
-		printf "<td><input type=radio name=smbfs_cname_def value=1 %s> $text{'linux_auto'}\n",
-			defined($options{"clientname"}) ? "" : "checked";
-		printf "<input type=radio name=smbfs_cname_def value=0 %s>\n",
-			defined($options{"clientname"}) ? "checked" : "";
-		print "<input size=10 name=smbfs_cname value=\"$options{clientname}\"></td> </tr>\n";
+		if ($support < 3) {
+			print "<td><b>$text{'linux_cname'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_cname_def value=1 %s> $text{'linux_auto'}\n",
+				defined($options{"clientname"}) ? "" : "checked";
+			printf "<input type=radio name=smbfs_cname_def value=0 %s>\n",
+				defined($options{"clientname"}) ? "checked" : "";
+			print "<input size=10 name=smbfs_cname value=\"$options{clientname}\"></td> </tr>\n";
 
-		print "<tr> <td><b>$text{'linux_mname'}</b></td>\n";
-		printf "<td colspan=3><input type=radio name=smbfs_mname_def value=1 %s> %s\n",
-			defined($options{"machinename"}) ? "" : "checked", $text{'linux_auto'};
-		printf "<input type=radio name=smbfs_mname_def value=0 %s>\n",
-			defined($options{"machinename"}) ? "checked" : "";
-		print "<input size=30 name=smbfs_mname value=\"$options{machinename}\"></td> </tr>\n";
-		}
-	
-	if ($support == 1) {
-		print "<tr> <td><b>$text{'linux_fmode'}</b></td>\n";
-		printf
-		    "<td><input name=smbfs_fmode size=5 value=\"%s\"></td>\n",
-		    defined($options{'fmode'}) ? $options{'fmode'} : "755";
+			print "<tr> <td><b>$text{'linux_mname'}</b></td>\n";
+			printf "<td colspan=3><input type=radio name=smbfs_mname_def value=1 %s> %s\n",
+				defined($options{"machinename"}) ? "" : "checked", $text{'linux_auto'};
+			printf "<input type=radio name=smbfs_mname_def value=0 %s>\n",
+				defined($options{"machinename"}) ? "checked" : "";
+			print "<input size=30 name=smbfs_mname value=\"$options{machinename}\"></td> </tr>\n";
+			}
+		
+		if ($support == 1) {
+			print "<tr> <td><b>$text{'linux_fmode'}</b></td>\n";
+			printf
+			    "<td><input name=smbfs_fmode size=5 value=\"%s\"></td>\n",
+			    defined($options{'fmode'}) ? $options{'fmode'} : "755";
 
-		print "<td><b>$text{'linux_dmode'}</b></td>\n";
-		printf
-		    "<td><input name=smbfs_dmode size=5 value=\"%s\"></td>\n",
-		    defined($options{'dmode'}) ? $options{'dmode'} : "755";
-		print "</tr>\n";
-		}
-	elsif ($support >= 3) {
-		print "<tr> <td><b>$text{'linux_fmode'}</b></td> <td>\n";
-		printf"<input type=radio name=smbfs_fmask_def value=1 %s> %s\n",
-			defined($options{'fmask'}) ? "" : "checked",
-			$text{'default'};
-		printf"<input type=radio name=smbfs_fmask_def value=0 %s>\n",
-			defined($options{'fmask'}) ? "checked" : "";
-		printf "<input name=smbfs_fmask size=5 value='%s'></td>\n",
-			$options{'fmask'};
+			print "<td><b>$text{'linux_dmode'}</b></td>\n";
+			printf
+			    "<td><input name=smbfs_dmode size=5 value=\"%s\"></td>\n",
+			    defined($options{'dmode'}) ? $options{'dmode'} : "755";
+			print "</tr>\n";
+			}
+		elsif ($support >= 3) {
+			print "<tr> <td><b>$text{'linux_fmode'}</b></td> <td>\n";
+			printf"<input type=radio name=smbfs_fmask_def value=1 %s> %s\n",
+				defined($options{'fmask'}) ? "" : "checked",
+				$text{'default'};
+			printf"<input type=radio name=smbfs_fmask_def value=0 %s>\n",
+				defined($options{'fmask'}) ? "checked" : "";
+			printf "<input name=smbfs_fmask size=5 value='%s'></td>\n",
+				$options{'fmask'};
 
-		print "<td><b>$text{'linux_dmode'}</b></td> <td>\n";
-		printf"<input type=radio name=smbfs_dmask_def value=1 %s> %s\n",
-			defined($options{'dmask'}) ? "" : "checked",
-			$text{'default'};
-		printf"<input type=radio name=smbfs_dmask_def value=0 %s>\n",
-			defined($options{'dmask'}) ? "checked" : "";
-		printf "<input name=smbfs_dmask size=5 value='%s'></td></tr>\n",
-			$options{'dmask'};
+			print "<td><b>$text{'linux_dmode'}</b></td> <td>\n";
+			printf"<input type=radio name=smbfs_dmask_def value=1 %s> %s\n",
+				defined($options{'dmask'}) ? "" : "checked",
+				$text{'default'};
+			printf"<input type=radio name=smbfs_dmask_def value=0 %s>\n",
+				defined($options{'dmask'}) ? "checked" : "";
+			printf "<input name=smbfs_dmask size=5 value='%s'></td></tr>\n",
+				$options{'dmask'};
 
-		print "<tr> <td><b>$text{'linux_ro'}</b></td>\n";
-		printf "<td><input type=radio name=smbfs_ro value=1 %s> $text{'yes'}\n",
-			defined($options{"ro"}) ? "checked" : "";
-		printf "<input type=radio name=smbfs_ro value=0 %s> $text{'no'}</td>\n",
-			defined($options{"ro"}) ? "" : "checked";
+			print "<tr> <td><b>$text{'linux_ro'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_ro value=1 %s> $text{'yes'}\n",
+				defined($options{"ro"}) ? "checked" : "";
+			printf "<input type=radio name=smbfs_ro value=0 %s> $text{'no'}</td>\n",
+				defined($options{"ro"}) ? "" : "checked";
 
-		print "<td><b>$text{'linux_user'}</b></td>\n";
-		printf "<td><input type=radio name=smbfs_user2 value=1 %s> $text{'yes'}\n",
-			defined($options{"user"}) ? "checked" : "";
-		printf "<input type=radio name=smbfs_user2 value=0 %s> $text{'no'}</td> </tr>\n",
-			defined($options{"user"}) ? "" : "checked";
-		}
-	if ($support == 4) {
-		print "<tr> <td><b>$text{'linux_cname'}</b></td>\n";
-		printf "<td colspan=3><input type=radio name=smbfs_cname_def value=1 %s> $text{'linux_auto'}\n",
-			defined($options{"netbiosname"}) ? "" : "checked";
-		printf "<input type=radio name=smbfs_cname_def value=0 %s>\n",
-			defined($options{"netbiosname"}) ? "checked" : "";
-		print "<input size=40 name=smbfs_cname value=\"$options{netbiosname}\"></td> </tr>\n";
+			print "<td><b>$text{'linux_user'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_user2 value=1 %s> $text{'yes'}\n",
+				defined($options{"user"}) ? "checked" : "";
+			printf "<input type=radio name=smbfs_user2 value=0 %s> $text{'no'}</td> </tr>\n",
+				defined($options{"user"}) ? "" : "checked";
+			}
+		if ($support == 4) {
+			print "<tr> <td><b>$text{'linux_cname'}</b></td>\n";
+			printf "<td colspan=3><input type=radio name=smbfs_cname_def value=1 %s> $text{'linux_auto'}\n",
+				defined($options{"netbiosname"}) ? "" : "checked";
+			printf "<input type=radio name=smbfs_cname_def value=0 %s>\n",
+				defined($options{"netbiosname"}) ? "checked" : "";
+			print "<input size=40 name=smbfs_cname value=\"$options{netbiosname}\"></td> </tr>\n";
 
-		print "<tr> <td><b>$text{'linux_mname'}</b></td>\n";
-		printf "<td colspan=3><input type=radio name=smbfs_mname_def value=1 %s> %s\n",
-			defined($options{"ip"}) ? "" : "checked", $text{'linux_auto'};
-		printf "<input type=radio name=smbfs_mname_def value=0 %s>\n",
-			defined($options{"ip"}) ? "checked" : "";
-		print "<input size=40 name=smbfs_mname value=\"$options{ip}\"></td> </tr>\n";
+			print "<tr> <td><b>$text{'linux_mname'}</b></td>\n";
+			printf "<td colspan=3><input type=radio name=smbfs_mname_def value=1 %s> %s\n",
+				defined($options{"ip"}) ? "" : "checked", $text{'linux_auto'};
+			printf "<input type=radio name=smbfs_mname_def value=0 %s>\n",
+				defined($options{"ip"}) ? "checked" : "";
+			print "<input size=40 name=smbfs_mname value=\"$options{ip}\"></td> </tr>\n";
+
+			print "<tr> <td><b>$text{'linux_wg'}</b></td>\n";
+			printf "<td><input type=radio name=smbfs_wg_def value=1 %s> $text{'linux_auto'}\n",
+				defined($options{"workgroup"}) ? "" : "checked";
+			printf "<input type=radio name=smbfs_wg_def value=0 %s>\n",
+				defined($options{"workgroup"}) ? "checked" : "";
+			print "<input size=10 name=smbfs_wg value=\"$options{'workgroup'}\"></td>\n";
+
+			print "</tr>\n";
+			}
 		}
 	}
 elsif ($_[0] eq "reiserfs") {
@@ -1529,14 +1611,14 @@ if (($_[0] eq "nfs") || ($_[0] eq "nfs4")) {
 		if ($in{nfs_host} !~ /^\S+$/) {
 			&error(&text('linux_ehost', $in{'nfs_host'}));
 			}
-		$out = `ping -c 1 '$in{nfs_host}' 2>&1`;
+		$out = &backquote_command("ping -c 1 '$in{nfs_host}' 2>&1");
 		if ($out =~ /unknown host/) {
 			&error(&text('linux_ehost2', $in{'nfs_host'}));
 			}
 		elsif ($out =~ /100\% packet loss/) {
 			&error(&text('linux_edown', $in{'nfs_host'}));
 			}
-		$out = `showmount -e '$in{nfs_host}' 2>&1`;
+		$out = &backquote_command("showmount -e '$in{nfs_host}' 2>&1");
 		if ($out =~ /Unable to receive/) {
 			&error(&text('linux_enfs', $in{'nfs_host'}));
 			}
@@ -1556,25 +1638,23 @@ if (($_[0] eq "nfs") || ($_[0] eq "nfs4")) {
 	    }
 
 	# Try a test mount to see if filesystem is available
-	$temp = &tempname();
-	mkdir($temp, 0755);
-	$mout = `mount -t $_[0] $in{nfs_host}:$in{nfs_dir} $temp 2>&1`;
+	$temp = &transname();
+	&make_dir($temp, 0755);
+	&execute_command("mount -t $_[0] $in{nfs_host}:$in{nfs_dir} $temp",
+			 undef, \$mout, \$mout);
 	if ($mout =~ /No such file or directory/i) {
-		rmdir($temp);
 		&error(&text('linux_enfsdir', $in{'nfs_dir'},
 			     $in{'nfs_host'}, "<pre>$dirlist</pre>"));
 		}
 	elsif ($mout =~ /Permission denied/i) {
-		rmdir($temp);
 		&error(&text('linux_enfsperm', $in{'nfs_dir'}, $in{'nfs_host'}));
 		}
 	elsif ($?) {
-		rmdir($temp);
 		&error(&text('linux_enfsmount', "<tt>$mout</tt>"));
 		}
 	# It worked! unmount
-	system("umount $temp >/dev/null 2>&1");
-	rmdir($temp);
+	&execute_command("umount $temp >/dev/null 2>&1");
+	&unlink_file($temp);
 
 	return "$in{nfs_host}:$in{nfs_dir}";
 	}
@@ -1849,98 +1929,100 @@ elsif ($_[0] eq $smbfs_fs || $_[0] eq "cifs") {
 		$options{$support == 4 ? 'password' : 'passwd'} = $in{smbfs_passwd};
 		}
 
-	if ($support != 2) {
-		delete($options{uid});
-		if ($in{smbfs_uid} ne "") { $options{uid} = getpwnam($in{smbfs_uid}); }
+	if (!$access{'simopts'}) {
+		if ($support != 2) {
+			delete($options{uid});
+			if ($in{smbfs_uid} ne "") { $options{uid} = getpwnam($in{smbfs_uid}); }
 
-		delete($options{gid});
-		if ($in{smbfs_gid} ne "") { $options{gid} = getgrnam($in{smbfs_gid}); }
-		}
+			delete($options{gid});
+			if ($in{smbfs_gid} ne "") { $options{gid} = getgrnam($in{smbfs_gid}); }
+			}
 
-	if ($support == 1) {
-		delete($options{servername});
-		if (!$in{smbfs_sname_def})
-			{ $options{servername} = $in{smbfs_sname}; }
-		}
-	elsif ($support == 2) {
-		delete($options{workgroup});
-		if (!$in{smbfs_wg_def})
-			{ $options{workgroup} = $in{smbfs_wg}; }
-		}
+		if ($support == 1) {
+			delete($options{servername});
+			if (!$in{smbfs_sname_def})
+				{ $options{servername} = $in{smbfs_sname}; }
+			}
+		elsif ($support == 2 || $support == 4) {
+			delete($options{workgroup});
+			if (!$in{smbfs_wg_def})
+				{ $options{workgroup} = $in{smbfs_wg}; }
+			}
 
-	if ($support < 3) {
-		delete($options{clientname});
-		if (!$in{smbfs_cname_def})
-			{ $options{clientname} = $in{smbfs_cname}; }
+		if ($support < 3) {
+			delete($options{clientname});
+			if (!$in{smbfs_cname_def})
+				{ $options{clientname} = $in{smbfs_cname}; }
 
-		delete($options{machinename});
-		if (!$in{smbfs_mname_def})
-			{ $options{machinename} = $in{smbfs_mname}; }
-		elsif (!gethostbyname($in{'smbfs_server'})) {
-			# No hostname found for the server.. try to guess
-			local($out, $sname);
-			$sname = $in{'smbfs_server'};
-			$out = `$config{'nmblookup_path'} -d 0 $sname 2>&1`;
-			if (!$? && $out =~ /^([0-9\.]+)\s+$sname\n/) {
-				$options{machinename} = $1;
+			delete($options{machinename});
+			if (!$in{smbfs_mname_def})
+				{ $options{machinename} = $in{smbfs_mname}; }
+			elsif (!gethostbyname($in{'smbfs_server'})) {
+				# No hostname found for the server.. try to guess
+				local($out, $sname);
+				$sname = $in{'smbfs_server'};
+				$out = &backquote_command("$config{'nmblookup_path'} -d 0 $sname 2>&1");
+				if (!$? && $out =~ /^([0-9\.]+)\s+$sname\n/) {
+					$options{machinename} = $1;
+					}
 				}
 			}
-		}
-	elsif ($support == 4) {
-		delete($options{"netbiosname"});
-		if (!$in{"smbfs_cname_def"}) {
-			$in{"smbfs_cname"} =~ /^\S+$/ ||
-				&error($text{'linux_ecname'});
-			$options{"netbiosname"} = $in{"smbfs_cname"};
-			}
-		delete($options{"ip"});
-		if (!$in{"smbfs_mname_def"}) {
-			gethostbyname($in{"smbfs_mname"}) ||
-			    &check_ipaddress($in{"smbfs_mname"}) ||
-				&error($text{'linux_emname'});
-			$options{"ip"} = $in{"smbfs_mname"};
-			}
-		}
-
-	if ($support == 1) {
-		delete($options{fmode});
-		if ($in{smbfs_fmode} !~ /^[0-7]{3}$/) {
-			&error(&text('linux_efmode', $in{'smbfs_fmode'}));
-			}
-		elsif ($in{smbfs_fmode} ne "755")
-			{ $options{fmode} = $in{smbfs_fmode}; }
-
-		delete($options{dmode});
-		if ($in{smbfs_dmode} !~ /^[0-7]{3}$/) {
-			&error(&text('linux_edmode', $in{'smbfs_dmode'}));
-			}
-		elsif ($in{smbfs_dmode} ne "755")
-			{ $options{dmode} = $in{smbfs_dmode}; }
-		}
-	elsif ($support >= 3) {
-		if ($in{'smbfs_fmask_def'}) {
-			delete($options{'fmask'});
-			}
-		else {
-			$in{'smbfs_fmask'} =~ /^[0-7]{3}$/ ||
-			    &error(&text('linux_efmode', $in{'smbfs_fmask'}));
-			$options{'fmask'} = $in{'smbfs_fmask'};
+		elsif ($support == 4) {
+			delete($options{"netbiosname"});
+			if (!$in{"smbfs_cname_def"}) {
+				$in{"smbfs_cname"} =~ /^\S+$/ ||
+					&error($text{'linux_ecname'});
+				$options{"netbiosname"} = $in{"smbfs_cname"};
+				}
+			delete($options{"ip"});
+			if (!$in{"smbfs_mname_def"}) {
+				gethostbyname($in{"smbfs_mname"}) ||
+				    &check_ipaddress($in{"smbfs_mname"}) ||
+					&error($text{'linux_emname'});
+				$options{"ip"} = $in{"smbfs_mname"};
+				}
 			}
 
-		if ($in{'smbfs_dmask_def'}) {
-			delete($options{'dmask'});
-			}
-		else {
-			$in{'smbfs_dmask'} =~ /^[0-7]{3}$/ ||
-			    &error(&text('linux_edmode', $in{'smbfs_dmask'}));
-			$options{'dmask'} = $in{'smbfs_dmask'};
-			}
+		if ($support == 1) {
+			delete($options{fmode});
+			if ($in{smbfs_fmode} !~ /^[0-7]{3}$/) {
+				&error(&text('linux_efmode', $in{'smbfs_fmode'}));
+				}
+			elsif ($in{smbfs_fmode} ne "755")
+				{ $options{fmode} = $in{smbfs_fmode}; }
 
-		delete($options{'ro'}); delete($options{'rw'});
-		if ($in{'smbfs_ro'}) { $options{'ro'} = ''; }
+			delete($options{dmode});
+			if ($in{smbfs_dmode} !~ /^[0-7]{3}$/) {
+				&error(&text('linux_edmode', $in{'smbfs_dmode'}));
+				}
+			elsif ($in{smbfs_dmode} ne "755")
+				{ $options{dmode} = $in{smbfs_dmode}; }
+			}
+		elsif ($support >= 3) {
+			if ($in{'smbfs_fmask_def'}) {
+				delete($options{'fmask'});
+				}
+			else {
+				$in{'smbfs_fmask'} =~ /^[0-7]{3}$/ ||
+				    &error(&text('linux_efmode', $in{'smbfs_fmask'}));
+				$options{'fmask'} = $in{'smbfs_fmask'};
+				}
 
-		delete($options{'user'});
-		if ($in{'smbfs_user2'}) { $options{'user'} = ''; }
+			if ($in{'smbfs_dmask_def'}) {
+				delete($options{'dmask'});
+				}
+			else {
+				$in{'smbfs_dmask'} =~ /^[0-7]{3}$/ ||
+				    &error(&text('linux_edmode', $in{'smbfs_dmask'}));
+				$options{'dmask'} = $in{'smbfs_dmask'};
+				}
+
+			delete($options{'ro'}); delete($options{'rw'});
+			if ($in{'smbfs_ro'}) { $options{'ro'} = ''; }
+
+			delete($options{'user'});
+			if ($in{'smbfs_user2'}) { $options{'user'} = ''; }
+			}
 		}
 	}
 elsif ($_[0] eq "reiserfs") {
@@ -2012,6 +2094,17 @@ elsif ($_[0] eq "ntfs") {
 	if ($in{ntfs_gid} ne "")
 		{ $options{"gid"} = getgrnam($in{ntfs_gid}); }
 	}
+elsif ($_[0] eq "swap") {
+	# Save SWAP options
+	if ($in{'swap_pri_def'}) {
+		delete($options{'pri'});
+		}
+	else {
+		$in{'swap_pri'} =~ /^\d+$/ && $in{'swap_pri'} <= 32767 ||
+			&error($text{'linux_eswappri'});
+		$options{'pri'} = $in{'swap_pri'};
+		}
+	}
 
 # Add loop option if mounting a normal file
 if ($_[0] ne "swap" && $_[0] ne "auto" && $_[0] ne "autofs" &&
@@ -2054,11 +2147,11 @@ close(SMBOPTS);
 sub write_smbopts
 {
 local($_);
-open(SMBOPTS, "> $module_config_directory/smbfs");
+&open_tempfile(SMBOPTS, "> $module_config_directory/smbfs");
 foreach (keys %smbopts) {
-	print SMBOPTS "$_\t$smbopts{$_}\n";
+	&print_tempfile(SMBOPTS, "$_\t$smbopts{$_}\n");
 	}
-close(SMBOPTS);
+&close_tempfile(SMBOPTS);
 }
 
 
@@ -2067,12 +2160,13 @@ close(SMBOPTS);
 sub create_swap
 {
 local($out, $bl);
-$bl = $_[1] * ($_[2] eq "m" ? 1024 : 1);
-$out = `dd if=/dev/zero of=$_[0] bs=1024 count=$bl 2>&1`;
+$bl = $_[1] * ($_[2] eq "g" ? 1024*1024 :
+	       $_[2] eq "m" ? 1024 : 1);
+$out = &backquote_logged("dd if=/dev/zero of=$_[0] bs=1024 count=$bl 2>&1");
 if ($?) { return "dd failed : $out"; }
-$out = `mkswap $_[0] $bl 2>&1`;
+$out = &backquote_logged("mkswap $_[0] $bl 2>&1");
 if ($?) { return "mkswap failed : $out"; }
-system("sync >/dev/null 2>&1");
+&system_logged("sync >/dev/null 2>&1");
 return 0;
 }
 
@@ -2083,7 +2177,7 @@ sub exports_list
 {
 local($dref, $cref, $out, $_);
 $dref = $_[1]; $cref = $_[2];
-$out = `showmount -e $_[0] 2>&1`;
+$out = &backquote_command("showmount -e ".quotemeta($_[0])." 2>&1", 1);
 if ($?) { return $out; }
 
 # Add '/' if the server is in NFSv4
@@ -2104,7 +2198,7 @@ sub nfs_max_version
 {
     local($_, $max, $out);
     $max = 0;
-    $out = `/usr/sbin/rpcinfo -p $_[0] 2>&1`;
+    $out = &backquote_command("/usr/sbin/rpcinfo -p ".quotemeta($_[0])." 2>&1", 1);
     if ($?) { return $out; }
     foreach (split(/\n/, $out)) {
 	if ((/ +(\d) +.*nfs/) && ($1 > $max)) {
@@ -2118,7 +2212,7 @@ sub nfs_max_version
 sub broadcast_addr
 {
 local($out);
-$out = `ifconfig -a 2>&1`;
+$out = &backquote_command("ifconfig -a 2>&1", 1);
 if ($out =~ /(eth|tr)\d\s+.*\n.*Bcast:(\S+)\s+/) { return $2; }
 return "255.255.255.255";
 }
@@ -2171,9 +2265,10 @@ return $rv;
 # write_amd_conf(text)
 sub write_amd_conf
 {
-open(AMD, ">$config{'auto_file'}");
-print AMD $_[0];
-close(AMD);
+local @af = split(/\s+/, $config{'auto_file'});
+&open_tempfile(AMD, ">$config{'auto_file'}");
+&print_tempfile(AMD, $_[0]);
+&close_tempfile(AMD);
 }
 
 # parse_amd_conf()
@@ -2245,6 +2340,8 @@ return $_[0] =~ /^\/dev\/(s|h)d([a-z])(\d+)$/ ?
 	&text('linux_label', "$1") :
        $_[0] eq '/dev/cdrom' ?
 	$text{'linux_cddev'} :
+       $_[0] eq '/dev/burner' ?
+	$text{'linux_burnerdev'} :
        $_[0] =~ /cdroms\/cdrom(\d+)$/ ?
 	&text('linux_cddev2', "$1") :
 	$_[0];
